#!/tmp/.ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZpErLZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZperl/bin/perl
use strict;
use PureMessage::Manager::Request qw($REQUEST %in %text);
use PureMessage::Manager::Config qw(%web_config save_web_config);
use PureMessage::Manager::Lang;
use PureMessage::Manager::Modules;
use PureMessage::Manager::Users qw(:all);
use PureMessage::Manager::Response::Services::Manager;
use PureMessage::Manager::Widget;
use PureMessage::Manager::Utils qw(html_escape);
use Time::Local;

my $r = $REQUEST->get_response( 'PureMessage::Manager::Response::Services::Manager',
                                'stderr_to_stdout' =>1
                              );
$r->add_help(text => "Help", tab => "locserv", page => "mgr_locserv_httpd_browse_act_log" );

my %all_users;

eval { open_db({}); %all_users = list_users(); };
if ($@) {
    raise_error($@);
    header("Error reading ACL table.");
    exit;
}

$r->title(text('log_index_title'));

unless ( -f $web_config{'activity_log'} ) {
    open(LOG, ">$web_config{'activity_log'}");
    close LOG;
}

if (not $web_config{'log'}) {
    $r->print_notification( text('log_index_nolog', 'edit_log.cgi'),"<p>\n" );
}
elsif (not $web_config{'logfiles'}) {
    $r->print_notification( text('log_index_nologfiles', 'edit_log.cgi'),"<p>\n" );
}

my $WIDGET = new PureMessage::Manager::Widget;
$WIDGET->read_input($REQUEST, 'query_log');
my $q = $WIDGET->result_data();

if ( $REQUEST->query('id') ) {
    show_action($REQUEST->query('id'));
}
elsif ( $q->{search} or $REQUEST->query('search') ) {
    if ( not get_results() ) {
        $r->print_notification(text('search_none'));
        show_query($r);
    }
    else {
        my %state = ( 'search' => 1 );
        foreach ( qw( from_m from_d from_y to_m to_d to_y uall mall ) ) {
            $state{$_} = $REQUEST->query($_) if $REQUEST->query($_);
        }
        
        my $b = button({type => 'image', icon=>'search',
                        href=>'activity.cgi',
                        text=>text('search_again')}, style=>"toolbar");

        $r->add_part($b."<br>");
        
        $r->addGrid(
            name => 'results',
            width => '100%',
            title => text('search_title'),
            columns => 8,
            data => \&get_results,
            paging => 1,
            page_size => 10,
            state => \%state,
            action => "activity.cgi",
            ddata => [
                        {
                        extra       => 'nowrap',
                        description => text('search_action')
                        },
                        {
                        width       => 200,
                        description => text('search_type')
                        },
                        {
                        extra       => 'nowrap',
                        description => text('search_object') 
                        },
                        {
                        extra       => 'nowrap',
                        description => text('search_module')
                        },
                        {
                        extra       => 'nowrap',
                        description => text('search_user')   
                        },
                        {
                        extra       => 'nowrap',
                        description => text('search_host')   
                        },
                        {
                        extra       => 'nowrap',
                        description => text('search_date')   
                        },
                        {
                        extra       => 'nowrap',
                        description => text('search_time')
                        },
                    ],

            );
    }
}
else {
    show_query($r);
}

$r->send();

sub show_action {
    my $id = shift;
    my $act = find_action($id);
    my ( $fields, $data ) = action_form($act);
    my $edit_log = $r->addForm(
                                name => "action",
                                data => $data,
                                fields => $fields,
                                columns => 2,
                                title => text('view_header', $act->{'id'}),
                                keys => sub { return ucfirst(shift)},
                               );
}

sub find_action {
    my $id = shift;
    open(LOG, $web_config{'activity_log'} ) 
        or return $r->raise_error(text('cant_open', $web_config{'activity_log'}, $!));
    my $act;
    while(<LOG>) {
        $act = $r->parse_logline($_);
        last if $act and $act->{'id'} eq $id;
    }
    close(LOG);
    return $act;
}

sub action_form {
    my $act = shift;
    my $fields =  [ 
                    text('view_action'),
                    text('search_type'),
                    text('search_object'),
                    text('view_module'),
                    text('view_user'),
                    text('view_ip'),  
                    text('view_time'),
                    text('view_script'),
                    text('view_sid'), undef,
                  ];
    my @tm = localtime($act->{'time'});
    my %ui = (
                    text('view_action')  => $act->{'action'},
                    text('search_type')  => $act->{'type'}, 
                    text('search_object')=> $act->{'object'},
                    text('view_module')  => sprintf("<a href='/%s'>%s</a>", $act->{'module'}, $act->{'module'}),
                    text('view_user')    => sprintf("<a href='index.cgi?user=%s'>%s</a>", $act->{'user'}, $act->{'user'}),
                    text('view_ip')      => $act->{'ip'},
                    text('view_time')    => sprintf ("%2.2d-%s-%4.4d %2.2d:%2.2d:%2.2d", $tm[3], 
                                                   text("smonth_".($tm[4]+1)), $tm[5]+1900,
                                                   $tm[2], $tm[1], $tm[0]),
                    text('view_script')  => sprintf("<a href='/%s'>%s</a>", "$act->{'module'}/$act->{'script'}", "$act->{'script'}"),
                    text('view_sid')     => sprintf("<a href='activity.cgi?search=1&sid=%s&uall=1&mall=1&tall=1'>%s</a>",
                                                $act->{'sid'}, $act->{'sid'}),
            );
    file_diffs($act, $fields, \%ui);
    return ($fields, \%ui);
}

sub file_diffs {
    my $act = shift;
    my $fields = shift;
    my $ui = shift;
    my $i=0;
    while(open(DIFF, "$ENV{'WEBMIN_VAR'}/diffs/$act->{'id'}.$i")) {
	my ($type, $object, $diff);
	my $line = <DIFF>;
	while (<DIFF>) { $diff .= $_; }
	close(DIFF);
        if ($line =~ /^(\/.*)/) {
            $type = 'modify'; 
            $object = $1;
        }
	elsif ($line =~ /^(\S+)\s+(.*)/) {
            $type = $1; $object = $2;
        }
	my $t = text("view_type_$type");
        push @$fields, "Action #". ($i+1);
        push @$fields, undef;
	if ($t =~ /\$2/ || !$diff) {
            $ui->{"Action #". ($i+1)} = text("view_type_$type", "<tt>$object</tt>", html_escape($diff));
        }
	else {
            $ui->{"Action #". ($i+1)} = &text("view_type_$type", "<tt>$object</tt>");
            $ui->{"Action #". ($i+1)}.= "<pre>".&html_escape($diff)."</pre>";
        }
        $i++;
    }
}

sub show_query {
    my $self = shift;
    my $tb = $self->ToolBar( buttons => [[{type=>'image', icon=>'search', name=>'search', text=>text('search')}]] );
    my ( $fields, $data ) = query_form();
    my $edit_log = $self->addForm(
                                name => "query_log",
                                data => $data,
                                fields => $fields,
                                columns => 2,
                                title => text('log_index_header'),
                                toolbar => $tb,
                                action => 'activity.cgi',
                                keys => sub { return ucfirst(shift)},
                               );
}

sub query_form {
    my $fields =  [ 
            text('from'), text('to'),
            text('log_index_user'), text('log_index_module'),
                  ];
    my %ui = (
            text('log_index_user') => $r->select_users(''),
            text('log_index_module') => $r->select_modules(''),
            text('from') => time_input('from'),
            text('to') => time_input('to'),
            );
    return ($fields, \%ui);
}

sub time_input
{
    my @tm = localtime(time());
    my $html = "";
    $html .= "<select name=$_[0]_d>";
    for(my $i=1; $i<=31; $i++) {
        my $selected = ($i == $tm[3]) ? "selected" : "";
        $html .= sprintf("<option value=%d $selected>%02d\n", $i, $i);
    }
    $html .= "</select>";
    $html .= "&nbsp;<select name=$_[0]_m>";
    for(my $i=0; $i<12; $i++) {
        my $selected = $i == $tm[4] ? "selected" : "";
        $html .= "<option value=$i $selected>".text("smonth_".($i+1))."\n";
    }
    $html .= "</select>";
    $html .= "&nbsp;<select name=$_[0]_y>";
    for(my $i=2003; $i<2020; $i++) {
        my $selected = $i == ($tm[5] + 1900) ? "selected" : "";
        $html .= "<option value=$i $selected>$i\n";
    }
    $html .= "</select>";
    return $html;
}

sub get_results {
    my ($s, $e) = @_;
    my @results;
    unless ( $r->{results} and ref($r->{results}) eq "ARRAY" ) {
        $r->{results} = [];
        open(LOG, $web_config{'activity_log'} ) 
            or return $r->raise_error(text('cant_open', $web_config{'activity_log'}, $!));
        $in{modules} = [$in{modules}] unless ref $in{modules} eq "ARRAY";
        my %mods = map { $_ => 1 } @{$in{modules}};
        $in{users} = [$in{users}] unless ref $in{users} eq "ARRAY";
        my %user = map { $_ => 1 } @{$in{users}};
        my $from = parse_time('from');
        my $to = parse_time('to');
        while(<LOG>) {
            my $act = $r->parse_logline($_);
            if ( $act and
                ($in{'uall'} || $user{$act->{'user'}} ) and
                ($in{'mall'} || $mods{$act->{'module'}}) and
                (!$in{'sid'} || $in{'sid'} eq $act->{'sid'}) and
                (!$from || $from < $act->{'time'}) and
                (!$to || $to > $act->{'time'})
                ) {

                my @tm = localtime($act->{'time'});
                push @results, [
                        "<a href='activity.cgi?id=$act->{'id'}'>" . $act->{'action'} . "</a>",
                        $act->{'type'} ? $act->{'type'} : '',
                        $act->{'object'} ? $act->{'object'} : '',
                        $act->{'module'},
                        $act->{'user'},
                        $act->{'ip'},
                        sprintf ("%2.2d-%s-%4.4d", $tm[3], text("smonth_".($tm[4]+1)), $tm[5]+1900),
                        sprintf ("%2.2d:%2.2d:%2.2d", $tm[2], $tm[1], $tm[0]),
                    ]
            }
        }
        close(LOG);
        @{$r->{results}} = reverse(@results);
    }
    if ( defined $s ) {
        return $r->{results};
    }
    else {
        return $#{$r->{results}};
    }
}

sub parse_time
{
    my $name = shift;
    my $d = $REQUEST->query($name."_d");
    my $m = $REQUEST->query($name."_m");
    my $y = $REQUEST->query($name."_y");
    return 0 if (!$d && !$y);
    my $rv;
    if ( $name eq "to" ) {
        eval { $rv = timelocal(59, 59, 23, $d, $m, $y-1900) };
    }
    else {
        eval { $rv = timelocal(0, 0, 0, $d, $m, $y-1900) };
    }
    $r->raise_error(text('search_etime')) if ($@);
    return $rv;
}


