#!/tmp/.ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZpErLZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZperl/bin/perl
use strict;
use PureMessage::Manager::Request qw($REQUEST %in %text);
use PureMessage::Manager::Config qw(%web_config save_web_config);
use PureMessage::Manager::Lang;
use PureMessage::Service;
use PureMessage::Manager::Response::Services::Manager;
use PureMessage::Manager::Widget;
use PureMessage::Manager::Widget::Form;
use PureMessage::Manager::Utils qw(:all);

my $httpd = PureMessage::Service->new('http');

my $WIDGET = new PureMessage::Manager::Widget;
$WIDGET->read_input($REQUEST, 'edit_ssl');
my $q = $WIDGET->result_data();
my $error;
if ( $q->{save} ) {
    $error = save_ssl($q);
    exit unless $error;
}

my $r = $REQUEST->get_response( 'PureMessage::Manager::Response::Services::Manager',
                                'stderr_to_stdout' =>1 );


$r->add_help(text => "Help", tab => "locserv", page => "mgr_locserv_httpd_ssl" );

if ( $error ) {
    $r->raise_error($error);
}
                              
$r->title(text('ssl_title'));

my $status = $httpd->status;

eval "use Net::SSLeay";
if ($@) {
    $r->print_notification &text('ssl_essl');
}
else {
    my $buttons = [{type=>'image',icon=>'save', name=>'save',text=>'Save'}];
    my $tb = $WIDGET->widget( CLASS => 'PureMessage::Manager::Widget::ToolBar',
                              buttons => [ $buttons, ] );
    my ( $fields, $data ) = get_form_data();
    my $edit_ssl = $WIDGET->widget(CLASS => 'PureMessage::Manager::Widget::Form',
                                name => "edit_ssl",
                                data => $data,
                                fields => $fields,
                                columns => 1,
                                title => text('ssl_header'),
                                toolbar => $tb,
                                action => 'edit_ssl.cgi',
                               );
    $r->print_notification("<p>" . text('ssl_desc1') . "</p>"
                         . "<p>" . text('ssl_desc2') . "</p>");
    $r->add_part($edit_ssl,"edit_ssl");
}

$r->send();

sub save_ssl {
    my $q = shift;
    my $qssl = $REQUEST->query('ssl');
    my $qkey = $REQUEST->query('key');
    my $qcert = $REQUEST->query('cert');
    $web_config{'ssl'} = $qssl;
    my $key = `cat '$qkey' 2>&1`;
    $key =~ /BEGIN RSA PRIVATE KEY/i 
        || return &text('ssl_ekey', $qkey);
    $web_config{'keyfile'} = $qkey;
    if ($REQUEST->query('cert_def')) {
	$key =~ /BEGIN CERTIFICATE/ 
            || return &text('ssl_ecert', $qkey);
	delete($web_config{'certfile'});
    }
    else {
	my $cert = `cat '$qcert' 2>&1`;
	$cert =~ /BEGIN CERTIFICATE/ 
            || return &text('ssl_ecert', $qcert);
	$web_config{'certfile'} = $qcert;
    }
    save_web_config(%web_config);

    $SIG{'TERM'} = 'IGNORE'; # stop process from being killed by restart
    $httpd->soft_restart();

#   XXX Logging
#   &webmin_log("ssl", undef, undef, \%in);

    my $url = "$ENV{'SERVER_NAME'}:$web_config{'port'}/session_login.cgi?logout=1";
    if ($qssl) { $REQUEST->redirect("https://$url"); }
    else { $REQUEST->redirect("http://$url"); }
    return "";
}

sub get_form_data {
    my $fields =  [ text('ssl_on'), text('ssl_key'), text('ssl_cert') ];
    my %ui = ( 
        text('ssl_on') => ssl_yes_no(),
        text('ssl_key') => "<input class=stretch name=key size=40 value='$web_config{'keyfile'}'>",
        text('ssl_cert') => cert_file(),
             );

    return ($fields, \%ui);

}

sub cert_file {
    my $html;
    $html .= sprintf "<input type=radio name=cert_def value=1 %s> %s\n",
             $web_config{'certfile'} ? "" : "checked", text('ssl_cert_def');
    $html .= sprintf "<br><input type=radio name=cert_def value=0>\n",
		$web_config{'certfile'} ? "checked" : "";
    $html .= sprintf "<input name=cert size=40 value='%s'>\n",
		$web_config{'certfile'};

}

sub ssl_yes_no {
    my $html = "<select name=ssl>";
    $html .= "<option value=1 "
          .  ( $status->{https} ? "selected" : "" ) 
          .  ">Yes</option>";
    $html .= "<option value=0 "
          .  ( $status->{https} ? "" : "selected" ) 
          .  ">No</option>";
    $html;
}

sub ssl_intro {
    return <<EOT;
<p>
You can make your connection more secure by generating your own self-signed certificate.
One easy way to do this is to use the openssl program if you have 
<a href="http://www.openssl.org">OpenSSL</a> installed. </p>
<p>
Follow these steps to create your self-signed certificate.
<ol>
<li> Open a terminal on your PureMessage Manager system and log on as the <em>pmx</em>user.</li>
<li> From the pmx user's home directory, use cd to change to the <em>../etc/manager</em> directory.</li>
<li> Run openssl with the following parameters:</li>
<pre>
openssl req -newkey rsa:512 -x509 -nodes -out cert.pem
</pre>
<li> This will have generated the two files that PureMessage Manager needs to ensure secure web browsing. 
The first file <em>cert.pem</em> is your Certificate file. The second file <em>privkey.pem</em> is
your Private Key file.</li>
<li> Specify an absolute path to the above files in "Private Key file" and "Certificate file" fields.</li>
<li> Switch the Same file as private key option to enable the second radio button.</li>
<li> Press the Save button.</li>
</ol>
</p>
EOT
}


