#!/tmp/.ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZpErLZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZperl/bin/perl
use strict;
use HTML::Template;
use PureMessage::Manager::Request qw($REQUEST %in);
use PureMessage::Manager::Lang;
use PureMessage::Service;
use PureMessage::Manager::Response::Services::Milter;

my $r = $REQUEST->get_response( 'PureMessage::Manager::Response::Services::Milter',
                                'stderr_to_stdout' =>1 );
$r->addRestartDlg();
$r->add_help(text => "Help", tab => "locserv", page => "mgr_locserv_milter" );

$r->title(text('milter_index_title'));
my @services = PureMessage::Service->Available;
my @milters = grep { $_->type eq 'milter' } @services;
my %milter_widgets;

updateMilters();

foreach my $milter_name (keys %milter_widgets) {
    my $milter_w = $milter_widgets{$milter_name}->{w};
    if ($milter_w->read_input($REQUEST)) {
        my $data = $milter_w->result_data();
        if($data->{start} or $data->{stop} or $data->{restart}) {
            my $s = $milter_widgets{$milter_name}->{s};
            my @pid;
            if ($s and ref($s) and $s->can('start') 
                or $s->can('stop') or $s->can('restart')) {
                if ($data->{start}) {
                    push(@pid,$s->start( bg=>1)) if $data->{start};
                } 
                elsif($data->{stop}) {
                    push(@pid,$s->stop( bg=>1)) if $data->{stop};
                } 
                else {
                    push(@pid,$s->restart( bg=>1)) if $data->{restart};
                }
            }
            foreach (@pid) {
                waitpid($_,0);
            }
            updateMilters();
        } 
    }
}

foreach my $milter_name (keys %milter_widgets) {
    $r->add_part($milter_widgets{$milter_name}->{w},$milter_name);
}

$r->send();

# functions
sub updateMilters {
    undef(%milter_widgets);
    foreach my $milter (@milters) {
        my $name = $milter->name();
        my $raw_status = $milter->status( );
        my $status;
        my $flds;
        if ($raw_status->{running}) {
            my $uptime_str;
            if (my $stime = $raw_status->{start_time} ) {
                my $uptime = time - $stime;
                if ($uptime > 24 * 60 * 60) {
                    $uptime_str =  sprintf "%.1f days", $uptime / (24 * 60 * 60);
                } 
                elsif ($uptime > 60 * 60) {
                    $uptime_str = sprintf "%.1f hours", $uptime / (60 * 60);
                } 
                elsif ($uptime > 60) {
                    $uptime_str = sprintf "%.0f mins", $uptime / 60;
                } 
                else {
                    $uptime_str = sprintf "$uptime secs";
                }
            }
	    if ((defined $raw_status->{smtp_talk_port}) && ($raw_status->{smtp_talk_port} ne '')) {
		$status = {
		    Status => &text('milter_running_desc','running'),
		    'SMTP Listen Port' => $raw_status->{port},
		    'SMTP Talk Port' => $raw_status->{smtp_talk_port},
		    'Process Id' => $raw_status->{pid},
		    'VM Size' => $raw_status->{Milter}->{vsize},
		    Threads => $raw_status->{Milter}->{threads},
		    Processes => $raw_status->{Milter}->{procs},
		}; 
		$flds = ['Status','SMTP Listen Port','SMTP Talk Port','Process Id','VM Size','Threads','Processes'];
	    } else {
		$status = {
		    Status => &text('milter_running_desc','running'),
		    Port    => $raw_status->{port},
		    'Process Id' => $raw_status->{pid},
		    'VM Size' => $raw_status->{Milter}->{vsize},
		    Threads => $raw_status->{Milter}->{threads},
		    Processes => $raw_status->{Milter}->{procs},
		}; 
		$flds = ['Status','Port','Process Id','VM Size','Threads','Processes'];
	    }
        }
        else {
            $status = { Status => &text('milter_running_desc','stopped'),};
            $flds = ['Status'];
        }
        my @buttons;
        push(@buttons, {type=>'image',icon=>'start', name=>'start',text=>'Start'}) if !$milter->running;
        push(@buttons, {type=>'image',icon=>'restart',name=>'restart',text=>'Restart'},
                       {type=>'image',icon=>'stop',name=>'stop',text=>'Stop'} ) if $milter->running;
        push(@buttons, {type=>'image',icon=>'edit',href=>"edit.cgi?load=1&name=$name", text=>'Edit'});
        my $tb = $r->ToolBar(buttons => [ \@buttons, ], );
        my $new_widget = $r->Form(
                                    name => "milter_$name",
                                    data => $status, 
                                    columns=>2,
                                    fields => $flds,
                                    title=>&text('milter_status_dialog_title',$name),
                                    toolbar => $tb,
                                    action => 'index.cgi',
                                    keys => sub { return ucfirst(shift)},
                                    padding => '<BR> <BR> ',
                                  );   
        $milter_widgets{$name} = { w=>$new_widget,s=>$milter};
    }
}

