#!/tmp/.ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZpErLZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZperl/bin/perl
use strict;
use HTML::Template;
use PureMessage::Manager::Config qw(%config);
use PureMessage::Version qw($PREFIX);
use PureMessage::Manager::Log qw(greplog readlog);
use Data::Dump;
use HTTP::Date;
use PureMessage::Manager::Request qw($REQUEST %in $in);
use PureMessage::Manager::Lang;
use PureMessage::Manager::Response::Services::Milter;
use PureMessage::Manager::Widget;
use ActiveState::Bytes qw(bytes_format);

my $WIDGET = new PureMessage::Manager::Widget;


my $r = $REQUEST->get_response( 'PureMessage::Manager::Response::Services::Milter',
                                'stderr_to_stdout' =>1
                              );

$r->add_help(text => "Help", tab => "locserv", page => "mgr_locserv_activity_log" );

our %in;
our $in;
# munge up the $in variable

our @errors;
my @notifications;

open (STDERR,">&STDOUT");


my %log2label = (
                 pmx_log => &text('label_pmx_log'),
                 pmx_err => &text('label_pmx_err'),
                 pmx_out => &text('label_pmx_out'),
                    );

# refresh header
my $s_file = $in{s_file} || 'pmx_log';
my $headers = ($in{c_refresh} and $in{c_refresh} =~ /^on/) ? "<meta HTTP-EQUIV=\"refresh\" CONTENT=\"5; URL=./logs.cgi?$in\">" : "";
$r->title( &text('log_title',$log2label{$s_file}) );
$r->add_head($headers);

my $filter = $in{'filter'};

my $pmx_conf_re = qr/^(\d) (\d\d\d\d-\d\dT\d\d:\d\d:\d\d[Zz]?) \[(\d+)\] (\w)*?: (\w)*?: (.*)/;

my $tb = $r->ToolBar( buttons => [[ { type=>'image', icon=>'search', text=>'View',name=>'s_update'},]] );

# template parameters
my $params = {
        FILTER => "$filter",
	s_view => [
		{VALUE => '20', SEL => '', LABEL => '20', },
		{VALUE => '50', SEL => 'selected', LABEL => '50', },
		{VALUE => '100', SEL => '', LABEL => '100', },
		{VALUE => '200', SEL => '', LABEL => '200', },
		{VALUE => '500', SEL => '', LABEL => '500', }
	],
	s_file => [
		{VALUE => 'pmx_log', SEL => 'selected', LABEL => $log2label{pmx_log}, },
		{VALUE => 'pmx_err', SEL => '', LABEL => $log2label{pmx_err}, },
		{VALUE => 'pmx_out', SEL => '', LABEL => $log2label{pmx_out}, },
	],		
	s_priority => [
		{VALUE => 0, SEL => 'selected', LABEL => 'All', },
		{VALUE => 1, SEL => '', LABEL => '(1) Info', },
		{VALUE => 2, SEL => '', LABEL => '(2) Notice', },
		{VALUE => 3, SEL => '', LABEL => '(3) Warning', },
		{VALUE => 4, SEL => '', LABEL => '(4) Error', },
	],
	c_refresh => '',
	c_collapse => 'checked',
	SECTION_HEADER=>$r->ThemeSectionHeader(title=>'Recent Activity')->output(),
	SECTION_FOOTER1=>$r->ThemeSectionFooter(toolbar=>$tb)->output(),
	SECTION_FOOTER2=>$r->ThemeSectionFooter()->output(),
};

sub pmx_error($) {
	print "<p><b>".html_escape($_[0])."</b></p>";
	exit;
}

sub log_read{
	my $filename = shift;
	my $offset = shift;
	my $nlines = shift;
	my $priority = shift || 0;
	my $regexps;
	
        $regexps = ["^$priority"] if $priority and $priority != -1;
	push(@$regexps,$filter) if $filter; # filter if we have filter data
        if ($regexps) {
            return greplog($filename,$offset,$nlines,1,$regexps);
	}
	else {
            return  readlog($filename,$offset,$nlines,1);
	}
}


sub log_process($$$$$) {
	my $buf = shift;
	my $offset = shift;
	my $nlines = shift;
	my $collapse = shift;
	my $priority = shift;
	my %message;
	my @actions = qw/accepted approved rejected discarded tempfail quarantine abort/;
	my $logreg = qr/^(.) (.*?) \[([^,]+)(,[^\]]+)?\] (.*)/;
	my $idreg = qr/^(\S*?): (.*)/;
	my $count = 0;
	
	#process log entries
	for (split ("\n", $buf)) {
            #return $count if ($count == $nlines);
            chomp;
            my $is_action = 0;	
            my ($code, $timestamp, $id, $is_message, $service, $rest, $pid);
            
            # -1 is priority for unformatted files
            if  ($priority == -1) {
                # skip pmx-mark entries
                (/^=== pmx-mark ===/) and next;
            }
            else {
                ($code, $timestamp, $pid, $service, $rest) = ($_ =~ $logreg);

                if ( not $timestamp = str2time($timestamp) ) {
                    # can't parse date - probably a good enough reason to skip this entry
                    #die "Malformed timestamp: '$timestamp'";
                    next;
                }
                
                # if we dont have valid id (like g8ONCWkE023709:), its not a message-related entry.
                ($id,my $idrest) = $rest =~ $idreg;	
                
                if ($id) {
                    $is_message = 1;
                    $rest = $idrest;
                    $message{$id}{timestamp} = $timestamp;	
                    $message{$id}{entries}++;
                    
                    #get action

                    ACTION: for my $action (@actions) {
                        ($rest =~ /\b$action\b/) and do {
                            $message{$id}{action} = $action;
                            $is_action++;
                            last ACTION;
                        };
                    }
                    
                    #if quarantined, find out reason
                    if ($is_action && ($message{$id}{action} eq 'quarantine')) {
                        $message{$id}{reason} = 'spam' if $rest =~ /\bSpam\b/;
                        $message{$id}{reason} = 'virus' if $rest =~ /\bVirus\b/;		
                    }
                }
                else {
                    $id = 'n/a';
                }
            }
            
            my $tmpl_index = $offset + $count;
            
            # populate template
            #$params->{t_log}->[$tmpl_index]->{ITEM_TD_CLASS} = ($count & 1) ? 'logeven' : 'logodd';
            
            # unformatted line
            if ($priority == -1) {
                $params->{t_log}->[$tmpl_index]->{ITEM_DESC_UNFORMATTED} = $_;
                $params->{t_log}->[$tmpl_index]->{ITEM_CLASS} = 'lognonmessage';
                $params->{t_log}->[$tmpl_index]->{ITEM_ID} = $count;
                $count++;		
            }
            else {
                # formatted non-message line
                my $time_string = scalar localtime($timestamp);
                $time_string =~ s/ /&nbsp;/g; #for better formatting
                if ( not ($is_message || $collapse)) {
                    my $tmpl_line = "$timestamp $pid $rest";
                    $params->{t_log}->[$tmpl_index]->{ITEM_DESC_TIME} = $time_string;
                    $params->{t_log}->[$tmpl_index]->{ITEM_DESC_PID} = $pid;
                    $params->{t_log}->[$tmpl_index]->{ITEM_DESC_MSGID} = $id;
                    $params->{t_log}->[$tmpl_index]->{ITEM_DESC_UNFORMATTED} = $rest;
                    $params->{t_log}->[$tmpl_index]->{ITEM_CLASS} = 'lognonmessage';
                    $params->{t_log}->[$tmpl_index]->{ITEM_ID} = $count;
                    $count++;			
                } 
                else {
                    # formatted message line
                    my $tmpl_line = "$time_string $pid $id";
                    if ($collapse) {
                        if ($is_action) {
                            # in collapse mode, only display the entries containing the action taken
                            $params->{t_log}->[$tmpl_index]->{ITEM_DESC_TIME} = $time_string;
                            $params->{t_log}->[$tmpl_index]->{ITEM_DESC_PID} = $pid;
                            $params->{t_log}->[$tmpl_index]->{ITEM_DESC_MSGID} = $id;
                            $params->{t_log}->[$tmpl_index]->{ITEM_DESC_UNFORMATTED} = $rest;
                            $params->{t_log}->[$tmpl_index]->{ITEM_CLASS} = $message{$id}{reason} ? 
                                                                                        'log'.$message{$id}{reason} : 
                                                                                        'log'.$message{$id}{action};
                            $params->{t_log}->[$tmpl_index]->{ITEM_ID} = $id;
                            $count++;
                        }
                    }
                    else {
                        $params->{t_log}->[$tmpl_index]->{ITEM_DESC_TIME} = $time_string;
                        $params->{t_log}->[$tmpl_index]->{ITEM_DESC_PID} = $pid;
                        $params->{t_log}->[$tmpl_index]->{ITEM_DESC_MSGID} = $id;
                        $params->{t_log}->[$tmpl_index]->{ITEM_DESC_UNFORMATTED} = $rest;
                        $params->{t_log}->[$tmpl_index]->{ITEM_ID} = $id;
                        
                        # if this is the "action" entry of a message
                        if ($is_action) {
                            my $cnt = 0; 
                            # go back and tag all the messages other entries
                            while ($message{$id}{entries}) {
                                if ($params->{t_log}->[$tmpl_index-$cnt]->{ITEM_ID} eq $id) {
                                        $params->{t_log}->[$tmpl_index-$cnt]->{ITEM_CLASS} = $message{$id}{reason} ? 
                                                                                            'log'.$message{$id}{reason} : 
                                                                                            'log'.$message{$id}{action};
                                        $message{$id}{entries}--;
                                }
                                $cnt++;
                            }
                        } 
                        $count++;
                    }	
                }
            }	
	}
        return $count;
}

# if a form was submitted
if (exists $in{s_update}) {	
	# update all fields
	for my $element (qw/s_view s_priority s_file/) {
		for my $node (@{$params->{$element}}) {
			$node->{SEL} = '';
			$node->{SEL} = 'selected' if $in{$element} and $in{$element} eq  $node->{VALUE};
		}
	}
	for my $element (qw/c_refresh c_collapse/) {
		$params->{$element} = ($in{$element} and $in{$element}=~/^on/) ? "checked" : "";
	}
}

my %file2config = (
	pmx_log => 'log_to',
	pmx_err => 'stderr',
	pmx_out => 'stdout',
);

#get some parameters
my $view_size = $in{s_view} || 50;	
my $filename = $config{$file2config{$s_file}};	
my $priority = $in{s_priority} || 0;

$priority = -1 if $s_file ne 'pmx_log'; #set special priority for non-log files

my $collapse = (($params->{c_collapse} eq  'checked') and !$priority);	# Collapse only valid for "All"

my $file_exists = 1;
#get log file stats
unless (-f $filename) {
    $file_exists = 0;
    push @notifications, text("log_no_".$s_file);
}
if ($file_exists) {
    unless (-r $filename) {
        $file_exists = 0;
        $r->raise_error("error_log_file_not_readable", $filename, $ENV{USER});
    }
}

if ($file_exists) {
    #read and process log
    my ($size,$mtime,$ctime) = (stat("$filename"))[7,9,10];
    $params->{LOG_STATS} = "[Size: ".bytes_format($size)." | Last modified: ".localtime($mtime)."]";
    my ($lines_read, $lines_processed) = (0,0);
    my $read_size = $collapse ? $view_size * 10 : $view_size;
    while ($lines_processed < $view_size) {    
        my $buf = log_read($filename, $lines_read, $read_size, $priority);
        my $new_lines = ($buf =~ tr/\n/\n/);  #count lines retrieve
        if ($new_lines) {
            $lines_processed += log_process($buf, $lines_processed, $view_size-$lines_processed, $collapse, $priority);
            $lines_read += $new_lines;
        }
        else {
            if (!$lines_processed) {
                if ($priority && ($priority != -1)) {
                        $params->{FOOTER} = notification_str("No priority $priority entries found.");
                }
                else {
                        $params->{FOOTER} = notification_str("No entries found.");
                }
            }
            else {
                    #$params->{FOOTER} = "End of file reached.";
            }
            last;
        } 
        if ($new_lines < $view_size) {
            if (($priority==-1) && !$lines_processed){
                $params->{FOOTER} = notification_str("No entries found.");
            }
            elsif (($priority==0) && !$lines_processed) {
                if ($collapse) {
                    $params->{FOOTER} = notification_str("No message-related entries found.");
                }
                else {
                    $params->{FOOTER} = notification_str("No entries found.");
                }
            }
            else {
                #$params->{FOOTER} = "End of file reached";
            }
            last;
        }
    }
    if ($lines_processed) {
        $params->{DO_LOG} = 1;    
        # chop off excess lines (from the front)
        if ($view_size < $lines_processed) {
            @{$params->{t_log}} = splice @{$params->{t_log}},@{$params->{t_log}}-$view_size, $view_size;
        }
    }
}
#print "<p>", error_dlg_str(), "</p>" if @errors;

print "<p>", notification_str($_), "</p>"  for @notifications;

#print the page
if ($file_exists) {
    #my $template = HTML::Template->new(filename => 'tmpl/logs.html');
    #$template->param($params);
    $r->add_template( file=>'tmpl/logs.html', data=>$params); 
}

$r->send();

sub notification_str {
        my $string = shift;
        return $WIDGET->widget( CLASS => 'PureMessage::Manager::Widget::PaneBox',
                                content => $string)->output();
}

	
__END__

