#!/tmp/.ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZpErLZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZperl/bin/perl
use strict;
use PureMessage::Manager::Request qw($REQUEST %in %text);
use PureMessage::Manager::Response::Services;
use PureMessage::Service;
use PureMessage::Manager::Lang;
use ActiveState::Config qw(:path);
use PureMessage::Scheduler::Config;
use PureMessage::Scheduler::Event qw(expand_time_range);

our %text;
my %vars;
my @raw_lines;
my %jobs;
my $input_ready = 0;
my $load = defined($in{'index'}) ? 1 : 0;
my $edit = ($load || $in{edit}) ? 1 : 0;

my %TYPE_MAP = (
    s =>  'sec',
    m =>  'min',
    h =>  'hour',
    md => 'mday',
    mo => 'mon',
    wd => 'wday',
);
my @ALLSPEC = qw(s m h md mo wd);

my $WIDGET = new PureMessage::Manager::Widget;

my $r = $REQUEST->get_response( 'PureMessage::Manager::Response::Services',
                                'stderr_to_stdout' =>1
                              );

$r->add_help(text => "Help", tab => "locserv", page => "mgr_locserv_sched" );

$r->title($edit ? &text('cron_edit_title') : text('cron_add_title'));

my @now = localtime(time);

my %data = ( time_frame     => 'advanced', # or repeating
             time_repeating => '', # in GUI as EVERY time_repeating time_frame_repeating
             time_frame_repeating => '',
             s_second   => $now[0],
             s_minute   => $now[1],
             s_hour     => $now[2],
             s_mday     => $now[3],
             s_month    => $now[4], # Starting on elements
             s_year     => $now[5] + 1900,
             a_sec      => '*',
             a_min      => '*', # advanced inputs
             a_hour     => '*',
             a_mday     => '*',
             a_mon      => '*',
             a_wday     => '*',
           );

# generate dropdown data for all our lists
my %OPTIONS;
{
    foreach my $t (qw(s m h md)) {
	my $r = expand_time_range('*',$t);
	my %map = map {$_ => sprintf('%02d',$_)} @$r;
	$OPTIONS{$t} = \%map;
	$OPTIONS{$t}{'*'} = 'Any';
    }

    my $i = 0;
    my @months = qw(January February March April May June July 
	            August September October November December);
    $OPTIONS{mo}{$i++} = $_ foreach @months;
    $OPTIONS{mo}{'*'} = 'Any';

    $i = 0;
    my @wdays = qw(Sunday Monday Tuesday Wednesday Thursday Friday Saturday);
    $OPTIONS{wd}{$i++} = $_ foreach @wdays;
    $OPTIONS{wd}{'*'} = 'Any';
}


if ($in{save} and readInput() and validInput(\%in)) {

    my %save_data = (
	action  => $vars{cmd}, 
	enabled => $vars{enabled} ? 1 : 0,
	desc    => $vars{comment} || "(no description)",
        type    => $vars{type}    || "exec",
    );
    
    my %when;
    foreach my $t (@ALLSPEC) {
	my $var = 'a_' . $TYPE_MAP{$t};
	if (ref($data{$var}) eq 'ARRAY') {
	    $when{$t} = join(',',@{$data{$var}});
	} else {
	    $when{$t} = $data{$var};
	}
    }

    my $jid = $vars{line};
    eval {

	# Try to auto-create an ID for this job.
	my $cf = PureMessage::Scheduler::Config->new(writable => 1);
	unless ($jid) {
	    my $cmd = (split(/ /, $save_data{action}))[0];
            $cmd =~ s/\W/_/g;
	    for (0..100) {
		my $t_jid = $cmd . "_" . $_;
		if (my $tmp = path_node($cf->root, "event#$t_jid", 
		    create => 'group')) {
		    unless (path_value($tmp, "id") || path_value($tmp, "action")) {
			$jid = $t_jid;
			last;
		    }
		}
	    }
	    # fall back to using time if all else fails.
	    unless ($jid) {
		$jid = "$cmd" . localtime;
	    }
	}
	
	my $n = path_node($cf->root, "event#$jid", create => 'group');
	if ($n) {
	    foreach my $k (qw(desc enabled type action)) {
		path_value($n, $k, $save_data{$k}) 
		    if exists $save_data{$k};
	    }
	    foreach my $k (@ALLSPEC) {
		my $v = $when{$k};
		if (defined($v) && $v !~ /^\s*\*?\s*$/) {
		    $v =~ s/^\s+//;
		    $v =~ s/\s+$//;
		    path_value($n, "when.$k", $v);
		} 
		else {
		    # XXX: we should do this, but Config is bugged
		    # path_node($n, "when.$k", 'delete'=>1);
		    path_value($n, "when.$k", '*');
		}
	    }
	}
	else {
	    # XXX hard coded string & no error details
	    $r->raise_error("Failed to create config node!");
	}
	$cf->save;
        successDialog();
    };
    if ($@) {
	# XXX more hard coded strings
	$r->raise_error("Failed!! $@");
    }
} 
else {
    putPage($load);
}

$r->send();

## functions

sub successDialog {
       $r->dialog( style => 'dlg_info',
            msg => &text("cron_updated",),
            title=>text('cron_updated_title'),
            buttons => [[ { type=>'image', text=>'Continue', icon=>'continue', href=>'index.cgi' } ]],
          );           
}

sub validInput {
        my $in = shift;
        $in->{comment} = '' unless $in->{comment}; # validator won't check keys that don't exists in input
        $in->{cmd} = '' unless $in->{cmd}; # validator won't check keys that don't exists in input
                                           # so force this to exist
        $REQUEST->typedecl("comment", "#value #nomarkup", "Description");
        $REQUEST->typedecl("cmd", "#value", "Command");
        my @v_errors = $REQUEST->validate($in);
        if (@v_errors) {
            $r->raise_error(join("<br>", @v_errors));
            return;
        } else {
            return 1;
        }
}
# generate an option list with the current value selected
# $data_hash has the format { form_element_name => 'text description' ..}
sub option_list {
        my ($name,$data_hash,$selected,$extra,$order) = @_;
        my @selected = ref($selected) eq 'ARRAY' ? @$selected : ($selected);
        $extra = '' unless $extra;
        my $return;
        my @order = @$order if $order;
        @order  = sort keys %$data_hash unless $order;
        $return = "<select name='$name' $extra>\n";
        foreach my $option (@order) {
                my $SELECTED;
                if (grep {$_ eq $option} @selected) {
                        $SELECTED =  'SELECTED';
                } else {
                        $SELECTED = '';
                }
                $return .= "<option value='$option' $SELECTED>$data_hash->{$option}\n";
        }
        $return .= "</select>\n";
}



sub loadInput {
        my $jid = $in{'index'};
        init_cron_data();
        
        $r->raise_error('error_bad_cron_line') && return 
                unless defined $jid;
        my %info = %{$jobs{$jid}};

        $r->raise_error('error_bad_cron_line') && return
                unless (%info and keys %info) and !$info{env} and $info{cmd};
        
        foreach ( qw( cmd data enabled) ) {
                $vars{$_} = $info{$_};
        }

        foreach (@ALLSPEC) {
	    my $t = $TYPE_MAP{$_};
	    $data{"a_$t"} = defined $info{$t} ? [split(',',$info{$t})] : ['*'];
        }
        $data{"time_spec"} = $vars{"time_spec"} = $info{time_spec};
        $vars{type} = $info{type};
        # housekeeping vars
        $vars{edit} = 1; 
        $vars{comment} = $info{COMMENT_DESCRIPTION} || "(no description)";
        $vars{line} = $jid;
        
        foreach (map {$TYPE_MAP{$_}} @ALLSPEC) {
                if (defined($info{$_}) and $info{$_} !~ /^\d+(,\d+)*$/) {
                   # unknown format we can't deal with it 
                   $vars{data_unknown} = 1;
                } 
        }
        
        # determine if we can parse it here!
        if (!$vars{data_unknown}) {
            $data{time_type} = $vars{time_type} = 'advanced';
        } else {
            $data{time_type} = $vars{time_type} = 'unknown';
        }
        
}

sub timeList {
        my ($step,$limit) = @_;
        return unless $limit;
        $step = 1 unless $step and $step > 0;
        my @a;
        for(my $i = 0; $i <= $limit; $i += $step) {
            push(@a,$i);
        }
        return \@a;
}

sub readInput {
        $input_ready = 1;
        foreach ( qw(cmd data enabled line type) ) {
            $vars{$_} = $in{$_};
        }
        foreach ( qw(a_min a_hour a_mday a_mon a_wday)) {
            $data{$_} = defined($in{$_}) ? $in{$_} : '*';
        }
        # the seconds field should default to 0 (safer)
        $data{a_sec} = defined($in{a_sec}) ? $in{a_sec} : "0";

        foreach ( qw(time_type time_spec) ) {
            $vars{$_} = $data{$_} = $in{$_};
        }
        
        $vars{comment} = $in{comment};
        $vars{line}    = $in{line};
        
        if ($data{time_type} eq 'unknown') {
            $vars{data_unknown} = $data{data_unknown} = 1;
            # parse the input line and see if it appears to be right
            my @t = split(' ',$data{time_spec});
            if ($#t != 5) { 
                $r->raise_error('bad_cron_format');
                return 0;
            } else {
                # move these into the appropriate slots
                $data{a_sec}  = $t[0] if defined $t[0];
                $data{a_min}  = $t[1] if defined $t[1];
                $data{a_hour} = $t[2] if defined $t[2];
                $data{a_mday} = $t[3] if defined $t[3];
                $data{a_mon}  = $t[4] if defined $t[4];
                $data{a_wday} = $t[5] if defined $t[5];
            }
        }
        
        return 1;
}
sub putPage {
        
        my $load = shift;
        
        if ($load) { loadInput();  } else { readInput() unless $input_ready; }

        if(!$edit) { $vars{enabled} = 1 unless scalar(keys %in); }
        
        my %state = ( 'line' => $vars{line},
                      'edit' => $edit,
                      'time_type' => $vars{time_type},
                      'type' => $vars{type},
                    );
        
        my $tb = $WIDGET->widget(CLASS =>'PureMessage::Manager::Widget::ToolBar',
                                 name => 'tb',
                                 buttons => [[ 
                                               { type=>'icon',icon=>'save',text=>'Save', name=>'save'},
                                               { type=>'icon',icon=>'cancel',text=>'Cancel', href=>'index.cgi'},
                                             ], 
                                             ],
                                 width=>600,
                                 state => \%state,
                                );

	foreach my $t (@ALLSPEC) {
	    my $r = expand_time_range('*',$t);
	    my $var = 'a_' . $TYPE_MAP{$t};
	    unshift @$r, '*';
	    my $options = $OPTIONS{$t};
	    $vars{$var} = option_list(
		    $var, $options, $data{$var}, 'size=9 multiple', $r,
		);
	}
        
        my $template = $WIDGET->widget( CLASS => 'PureMessage::Manager::Widget::Template',
                                   data => \%vars,
                                   filename => 'edit',
                                   );

        $vars{HEADER} = $r->ThemeSectionHeader( title=>&text('edit_cron_dialog_title',$in{hostname}))->output();
        $vars{FOOTER} = $r->ThemeSectionFooter( toolbar=>$tb)->output();
        $r->add_part($template,'body_form');

}


sub init_cron_data {

	my %c = PureMessage::Scheduler::Config->new->as_hash;
	my $events = $c{event};
	foreach my $evt_id (keys %$events) {
	    my $evt = $events->{$evt_id};
	    my @ts = qw(* * * * * *);
	    my %td;
	    if ($evt->{when}) {
		my %w = %{$evt->{when}};
		@ts = ();

		foreach my $t (@ALLSPEC) {
		    if (exists $w{$t} && $w{$t} ne '*') {
			my $r = expand_time_range($w{$t},$t);
			$td{$TYPE_MAP{$t}} = join(",", @$r);
			push(@ts, $w{$t});
		    }
		    else {
			push(@ts, "*");
		    }
		}
	    }
	    my $time_spec = join(" ", @ts);
	    
	    $jobs{$evt_id} =  {
		    id => $evt_id,
		    COMMENT_DESCRIPTION => $evt->{desc} || $evt->{action},
		    cmd => $evt->{action},
		    time_spec => $time_spec,
		    enabled => $evt->{enabled},
                    type => $evt->{type},
		    (%td),
		};
	}
}
