#!/tmp/.ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZpErLZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZperl/bin/perl
use strict;
use HTML::Template;
use PureMessage::Manager::Request qw($REQUEST %in %text);
use PureMessage::Manager::Response::Services;
use PureMessage::Manager::Widget;
use PureMessage::Manager::Widget::Grid;
use PureMessage::Service;
use PureMessage::Scheduler::Config;
use PureMessage::Scheduler::Event;
use PureMessage::Manager::Lang;
use ActiveState::Unix::Crontab qw(cron_list cron_parse cron_format cron_install);
use Time::Local;
use ActiveState::Duration qw(dur_format_eng);

# Figure out if the vscan scanner is running in daemon mode and ignore the error 
# if vscan is not installed.
my $no_daemons = 1;
eval { 
    require PureMessage::Policy::Virus::Daemon;
    $no_daemons = PureMessage::Policy::Virus::Daemon::no_daemons();
};

my $CURRENT_TIME = time;
my @mon = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);

my $WIDGET = new PureMessage::Manager::Widget;

my $r = $REQUEST->get_response( 'PureMessage::Manager::Response::Services',
                                'stderr_to_stdout' =>1
                              );

$r->add_taskmenu(
    { 
	title   => 'Perimeter Protection',
	buttons => [[ {type=>'text',text=>"Edit Log Watch Options", href=>"/Services/perimeter.cgi"} ]]
    }
);

$r->addRestartDlg();    
$r->add_help(text => "Help", tab => "locserv", page => "mgr_locserv_over" );

my $tb = $r->ToolBar(buttons=>[[{type=>'text', name=>'refresh', text=>'Ok'}]]);

my $REFRESH = { 
                5 => 'every 5 sec',
                10=> 'every 10 sec',
                20=> 'every 20 sec',
                30=> 'every 30 sec',
                0 => 'off',
              };
my @order = (5,10,20,30,0);

$r->add_taskmenu({title => 'Auto Refresh',
                  buttons => [[{type => 'custom', text => "<form style='margin: 0px; padding: 0px'>" 
                                                          . $WIDGET->option_list('refresh_int',$REFRESH,$in{refresh_int}||0,'',\@order) 
                                                          . $tb->output()
                                                          ."</form>"
                            }]],
                });

$r->title(text('services_index_title'));
my @services;
my @raw_lines;
my @jobs;

sub gen_service_data {
    my ($start,$end) = @_;
    my @data;
    if(defined($start) and defined($end) ) {
        for(my $i = $start; $i <= $end and $i <= $#services; $i++) {
            my $s = $services[$i];
            my($label,$status,$pid,$details);
            $status = defined $s->running ? ($s->running ? "running" : "stopped") : "unknown";
            if (text($s->type . '_label') ne $s->type . '_label') {
                $label = button({type=>'image', 
                                 text=>&text($s->type . '_label', $s->type, $s->name),
                                 href=>&text($s->type . '_href', $s->type, $s->name),
                                 icon=>$s->type . "_$status",
                                }, style=>"toolbar"); 
            } 
            else {
                $label = button({type=>'image', 
                                 text=> $s->name ? &text('generic_label2', $s->type, $s->name)
                                                 : &text('generic_label', $s->type),
                                 href=>&text('generic_href', $s->type, $s->name),
                                 icon=>"generic_$status",
                                }, style=>"toolbar"); 
            }
            my $stats = $s->status();
            $pid = $stats->{pid} || '-';
            $status = button({type => 'text', text=>$status});
            push(@data,[$label,$status,$pid,$stats->{summary}]);
        }
        return \@data;
    } 
    else {
        $#services;
    }
}

my $tb = $WIDGET->widget( CLASS => 'PureMessage::Manager::Widget::ToolBar',
                          buttons => [[ {type=>'image',icon=>'start', name=>'start', text=>'Start'},
                                        {type=>'image',icon=>'restart',name=>'restart', text=>'Restart' },
                                        {type=>'image',icon=>'stop',  name=>'stop', text=>'Stop' },
                                     ]],
                        );

my $service_widget = $WIDGET->widget(CLASS => 'PureMessage::Manager::Widget::Grid',
                            name => 'services_grid',
                            data => \&gen_service_data,
                            page => 0,
                            selectable=>1,
                            paging => 0,
                            title=>'Background Services',
                            columns=>4,
                            toolbar => $tb,
                            width=> "100%",
                            action=>'index.cgi',
                            padding=>'<BR> <BR>',
                            ddata => [
                                        { 
                                          width=>"20%",
                                          description=>'Service Type/Name',
                                          extra=>'nowrap',
                                        }, 
                                        { 
                                          width=>"10%",
                                          description=>'Status',
                                          extra=>'nowrap',
                                        }, 
                                        { 
                                          width=>"10%",
                                          description=>'PID',
                                        }, 
                                        { 
                                          width=>"60%",
                                          description=>'Details',
                                        }, 
                                     ],
                           );   


sub job_enable {
    my $enable = shift(@_) ? 1 : 0;
    my @list = @_;

    my $cf = PureMessage::Scheduler::Config->new(writable => 1);
    foreach my $jindex (@list) {
	my $j = $jobs[$jindex];
	$j->{enabled} = $enable;
	$cf->enable_event($j->{id})  if $enable == 1;
	$cf->disable_event($j->{id}) if $enable == 0;
    }
    $cf->save;
}

sub job_delete {
    my @list = @_;
    my $mod;

    my $cf = PureMessage::Scheduler::Config->new(writable => 1);
    foreach my $jindex (@list) {
	my $j = $jobs[$jindex];
	$cf->delete_event($j->{id});
    }
    delete($jobs[$_]) for @list;
    $cf->save;
}

sub get_job_button {
    my %job = @_;
    my $enabled = $job{enabled} ? "enabled" : "disabled";
    return button({ 
	    type => 'image', 
	    icon => "job_$enabled", 
	    href => "edit.cgi?index=$job{id}", 
	    text => $job{id} || $r->get_job_action($job{action}),
	}, 
	style => "toolbar"
    );
}

sub gen_jobs_data {
    my ($start,$end) = @_;
    my @data;
    if(defined($start) and defined($end) ) {
        for(my $i = $start; $i <= $end and $i <= $#jobs; $i++) {
	    next unless $jobs[$i];
            my %d = %{$jobs[$i]};
	    my $time_str = '-';
	    if ($d{enabled}) {
		$d{type} = 'null';
		my $event = eval { PureMessage::Scheduler::Event->new(
			id => $d{id},
			data => \%d,
			logger => undef,
		    )};
		if ($@) {
		    $time_str = '?';
		} else {
		    $event->calc_next_run($CURRENT_TIME);
		    my $delta_t = $event->{next_run} - $CURRENT_TIME;
		    $delta_t = 0 if $delta_t < 0;
		    $time_str = dur_format_eng($delta_t, '0.05', 'first');
		}
	    }
            push(@data, [
		get_job_button(%d),
		$d{enabled} ? "enabled" : "disabled", # XXX string constant
		$time_str,
		$d{desc} ? $d{desc} : $d{action},
		]);
        }
        return \@data;
    } 
    else {
        $#jobs;
    }
}

$tb = $WIDGET->widget( CLASS => 'PureMessage::Manager::Widget::ToolBar',
                       buttons => [[ {type=>'image',icon=>'enable', name=>'enable', text=>'Enable'},
                                     {type=>'image',icon=>'disable',  name=>'disable', text=>'Disable' },
                                     {type=>'image',icon=>'delete',  name=>'delete', text=>'Delete' },
                                     {type=>'image',icon=>'add',  href=>'edit.cgi', text=>'Add New' },
                                  ]],
                     );

my $jobs_widget = $WIDGET->widget(CLASS => 'PureMessage::Manager::Widget::Grid',
                            name => 'jobs_grid',
                            data => \&gen_jobs_data,
                            title=>'Scheduled Jobs',
                            page => 0,
                            selectable=>1,
                            paging => 0,
                            columns=>4,
                            toolbar => $tb,
                            width=>"100%",
                            action=>'index.cgi',
                            ddata => [
                                        { 
                                          width=>"20%",
                                          description=>'Job Name',
                                          extra=>'nowrap',
                                        }, { 
                                          width=>"10%",
                                          description=>'Status',
                                        }, { 
                                          width=>"15%",
                                          description=>'Next Run In',
                                          extra=>'nowrap=nowrap',
                                          align=>"left",
                                        }, { 
                                          width=>"55%",
                                          description=>'Description',
                                        }, 
                                     ],
                           );   


# init services list
@services = sort { lc(format_name($a->type,$a->name)) cmp lc(format_name($b->type,$b->name)) } PureMessage::Service->Available;

# hide the vscan service if there are enabled backends but none are running in daemon mode.
my $vscan_index;
for (my $i = 0; $i <= $#services; $i++) {
    my $s = $services[$i];
    if ($s->type() eq 'vscan' && $no_daemons) {
	$vscan_index = $i;
    }
}
splice(@services, $vscan_index, 1) if (defined $vscan_index);

# init job list
my %sconf = PureMessage::Scheduler::Config->new->as_hash;
my $events = $sconf{event};
@jobs = sort { $a->{action} cmp $b->{action} }
        map { $events->{$_}{id} = $_; $events->{$_} }
        grep { !$events->{$_}{trigger_only} }
        keys %$events;

sub format_name {
    my ($type,$name) = @_;
    PureMessage::Manager::Lang::set_namespace("Services");
    return text($type . '_label', $type, $name);
}

if ($service_widget->read_input($REQUEST)) {
    my $data = $service_widget->result_data();
    if ( exists($data->{selected_row}) and defined($data->{selected_row}) ) {
        my @selected;
        push(@selected,$data->{selected_row}) unless ref($data->{selected_row}) eq 'ARRAY';
        @selected = @{$data->{selected_row}} if ref($data->{selected_row}) eq 'ARRAY';
        
        if($data->{start} or $data->{stop} or $data->{restart}) {
            my @actions;
            my @pids;
            foreach my $row (@selected) {
                my $s = $services[$row];
                my $status = $s->status();
                if ($status->{read_only}) {
                    $r->raise_error(&text('no_start_stop',format_name($s->type,$s->name)));
                    next;
                }
		my $bg = $s->type eq "vscan" ? 0 : 1;
                
                if ($s and ref($s) and $s->can('start') and $data->{start}) {
                    if ( $s->running() > 0 ) {
                        $r->raise_error(text('already_running', format_name($s->type,$s->name)));
                    }
                    else {
                        eval {
                            push(@pids,$s->start(bg=>$bg));
                            push(@actions,&text('start',format_name($s->type,$s->name)))
                        };
                        if ($@) {
			    if ($@ =~ /^Start failed: no backends/) {
				$r->raise_error(text("vscan_no_backends"));
			    }
			    else {
				$r->raise_error($@);
			    }
                        }
                    }
                }
                if ($s and ref($s) and $s->can('stop') and $data->{stop}) {
                    if (!(  $s->type =~ /http/i  and $s->name =~ /Manager/)) {
                        eval {
                            push(@pids,$s->stop(bg=>$bg));
                            push(@actions,&text('stop',format_name($s->type,$s->name)))
                        };
                        if ($@) {
                            $r->raise_error($@);
                        }
                    }
                    else {
                        $r->raise_error(text('cannot_start_httpd'));
                    }
                }
                if ($s and ref($s) and $s->can('restart') and $data->{restart}) {
                    
                    $SIG{TERM} = 'IGNORE' if ( $s->type =~ /http/i and $s->name =~ /Manager/);

                    eval {
                        push(@pids,$s->restart(bg=>$bg));
                        push(@actions,&text('restart',format_name($s->type,$s->name)))
                    };
                    if ($@) {
			if ($@ =~ /^Start failed: no backends/) {
			    $r->raise_error(text("vscan_no_backends"));
			}
			else {
			    $r->raise_error($@);
			}
                    }
                }
                elsif($data->{restart}) {
                    $r->print_notification("'Restart' action is not supported for service <b>" . format_name($s->type,$s->name) . "</b>.");
                }
                if (@pids) {
                    do {
                            waitpid($_,0);
                       } foreach @pids;
                }
            }
            if(@actions) {
                foreach (@actions) {
                    $r->log($_, '', 'Action', \%in, 'Services/');
                }
                $r->add_part($WIDGET->widget(CLASS =>'PureMessage::Manager::Widget::PaneBox',
                                             content => join('<BR>',@actions),
                                             padding => '<BR> <BR> ',
                                            ),'actions'
                            );
            }
        }
    }
}

if ($jobs_widget->read_input($REQUEST)) {
    my $data = $jobs_widget->result_data();
    if ( exists($data->{selected_data_rows}) and defined($data->{selected_data_rows}) ) {
        my @selected = @{$data->{selected_data_rows}};
        if($data->{enable} or $data->{disable}) {
            job_enable($data->{enable} ? 1 : 0, @selected);
        } 
        elsif($data->{'delete'}) {
            job_delete(@selected);
        }
    }
} 

$r->add_part($service_widget,'main_body_service');
$r->add_part($jobs_widget,'main_body_jobs');

$r->send();

1;
