#!/tmp/.ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZpErLZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZperl/bin/perl

use HTML::Template;
use Data::Dumper;

use PureMessage::Manager::Request qw($REQUEST %in %text);
use PureMessage::Manager::Response::Support::Update qw( versionToDots);
use PureMessage::Manager::Lang;
use PureMessage::Manager::Widget;

my $RESPONSE;
my %vars;

$RESPONSE = $REQUEST->get_response( 'PureMessage::Manager::Response::Support::Update',
                                    'stderr_to_stdout' =>1 );

$RESPONSE->add_help(text => "Help", tab => "support", page => "mgr_support_upavail" );

my $WIDGET = new PureMessage::Manager::Widget;
my @upgrades;

# trap all errors as the HTTP content-type header
# has not been sent yet.
eval {

    # disable the ActiveState repository for this session as we
    # are only concerned with PureMessage modules.
    $RESPONSE->hide_as_repository();
    
    $WIDGET->read_input($REQUEST, 'upgrade');
    my $q = $WIDGET->result_data();

    # Upgrade all packages with updates
    if($q->{'all'}){
    	my $update = $RESPONSE->needUpdateList($REQUEST->query('repository'));
	my $updates = $$update{'updates'};
	foreach(keys(%{$updates})){
	    # Only upgrade it if it needs to be upgraded
            push(@upgrades, $_) if $$updates{$_};
	}
    }
};

if($@){
    $RESPONSE->title('upgrades_title');
    $RESPONSE->raise_error('general_error',$@);
    exit;
}

if(@upgrades){
    $RESPONSE->title('upgrade_tracking_title');
    $RESPONSE->display_install('upgrade', $REQUEST->query('repository'), @upgrades);
    $RESPONSE->send();
    exit;
}

$RESPONSE->title('upgrades_title');
my $template = HTML::Template->new(filename => 'tmpl/sm_upgrades.html', die_on_bad_params=>0);

# make the repository variable available in the template again if selected
# FIXME: the repository value should be checked for valid characters before being output again.
if($REQUEST->query('repository')){
    $template->param('REPOSITORY'=>$REQUEST->query('repository'));
}

# get the list of repositories that can be selected
my %repositories = $RESPONSE->ListOfRepositories();
my @rep_loop = ();
my %reverse_repo;
while(my($key, $val) = each(%repositories)){
	my %vars;
    $reverse_repo{$val} = $key;
	$vars{'REP_NAME'}=$key;
	$vars{'REP_LOC'}=$val;
	$vars{'SELECTED'} = 'selected' if $val eq $REQUEST->query('repository');
	push(@rep_loop,\%vars);
}

$template->param(REPOSITORIES=>\@rep_loop);
$RESPONSE->add_part($template,'body_part');

# Query for updates if a repository was specified.
if ($REQUEST->query('repository')){
    displayUpdateQuery($REQUEST->query('repository'));
}

$RESPONSE->send();

# Display all packages and indicate wich packages have updates available.
sub displayUpdateQuery{
    my $repository = $_[0];

    my $update = $RESPONSE->needUpdateList($repository);
    my $pkgs = $$update{'pkgs'};
    my $updates = $$update{'updates'};
	
    my @pkg;
    foreach(sort keys(%{$pkgs})){
        my $update_status;
        if(defined($$updates{$_})) {
          $update_status = $$updates{$_} ? text('update_available') : text('up_to_date');
        } else {
          $update_status = text('update_unknown');
        }
        
        push @pkg, [ $$updates{$_}
                     ? "<input type=checkbox name='upg_$_' checked disabled>"
                     : "",
                     $_, versionToDots($$pkgs{$_}{'VERSION'}),
                     $update_status,
                   ];
    }
    my $tb;
    if (%$updates and $reverse_repo{$repository} !~ /^PureMessage$/ ) {
        $template->param('ANY_UPDATES'=>1);
        $tb = $WIDGET->widget( CLASS => 'PureMessage::Manager::Widget::ToolBar',
                              buttons => [[ { type=>'text',
                                              text=>"Install All",
                                              name=>'all',
                                            },
                                         ]],
                                        );
    } else {
        $template->param('ANY_UPDATES'=>1);
    }
    print_table($RESPONSE, text('installed_title'), \@pkg, $tb);
    $template->param('QUERY'=>1);

}  

if ($@) {
  $RESPONSE->raise_error("$@");
  $RESPONSE->send();
}
sub print_table {
    my ($RESPONSE, $title, $data, $tb) = @_;

    sub highlightAvailable {
            my ($col,$index) = @_;
            if ($data->[$index]->[0] =~ /input/) {
                return "<span class='updateAvailable'>" . $col ."</span>";
            } else {
                return "$col";
            }
    }

    $RESPONSE->addGrid(
            title   => $title,
            name    => "upgrade",
            columns => 4,
            toolbar => $tb,
            data    => $data,
            width   => "100%",
            action  => "sm_upgrades.cgi",
            state   => {'repository' => $REQUEST->query('repository')},
            ddata   => [
                        {
                            width=>1,
                            extra=>'nowrap=nowrap',
                        },
                        {
                            description=>'Package',
                            extra=>'nowrap=nowrap',
                            decorate=> \&highlightAvailable,
                        },
                        {
                            description=>'Version',
                            extra=>'nowrap=nowrap',
                            decorate=> \&highlightAvailable,
                        },
                        {
                            description=>'Status',
                            extra=>'nowrap=nowrap',
                            decorate=> \&highlightAvailable,
                        },
                       ]
           );
}

1;
