#!/tmp/.ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZpErLZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZperl/bin/perl
# Dump the install-defaults.conf file to a network installation.
# Send the admin a courtesy email.
# Expects cgi parameter "role" to contain space-seperated list of
# roles that the remote host is installing.

use strict;
use warnings;
use PureMessage::Version qw($PREFIX);
use PureMessage::Config qw(%config);
use ActiveState::Config::Simple qw(config_parse_file);
use PureMessage::Install::Config qw(read_config write_config);
use PureMessage::Resources::DBH qw(register_resources_dsn);
use Mail::Send;
use CGI;

sub send_defaults;
sub send_404;
sub send_admin_email ($);

our $defaults = "$PREFIX/etc/install-defaults.conf";
our $cgi = new CGI;

if(-e $defaults) {
    send_defaults;
} else {
    send_404;    
}

exit(0);

sub send_defaults {
    my %roles = map { $_ => 1 } split(',', $cgi->param('role') || '');
    if ($roles{database}) {
        # we need to update our config with this database server, if it is new
        my $conf = config_parse_file($defaults);
        unless ($conf->{inst_opt}{database_host}) {
            my $host = $ENV{REMOTE_HOST};
            # no database host exists - so we should add this new host
            open(my $FH, ">>$defaults") or die "Can't open $defaults: $!";
            print $FH "<inst_opt database_host>\n"
                    . "    default = $host\n"
                    . "</inst_opt>\n";
            close $FH;

            my $instconf = "$PREFIX/etc/install.conf";
            if (-e $instconf) {
                # also update the install.conf
                my $ic = read_config($instconf);
                $ic->{inst_opts}{database_host} ||= $host;
                $ic = write_config( prefix => $PREFIX,
                                    opts => $ic->{inst_opts},
                                    pkgs => $ic->{pkgs},
                                    pmxuser => scalar(getpwuid($>)),
                                  );
            }
            
            register_resources_dsn("dbi:Pg:dbname=pmx_quarantine;port=5432;host=$host");
        }
    }

    my $dh;
    if(open($dh, $defaults)) {
        print $cgi->header(
            -content_type => 'text/x-as-config',
            -content_disposition => 'attachment; filename=install-defaults.conf'
        ), <$dh>;
        close($dh);
        send_admin_email(join ",", keys %roles);
    } else {
        die "Error reading $defaults: $!";
    }
}

sub send_404 {
    print $cgi->header(-status => 404), 
        "<html><body><h1>Not Found</h1>\n",
        "<p>This system does not appear to be the CSM.</p></body></html>\n";
}

sub send_admin_email ($) {
    my $role = shift;
    my $message = Mail::Send->new;
    $message->to($config{admin});
    $message->subject("PureMessage Network Installation Notification");
    my $fh = $message->open;
    print $fh <<"EOT";

The remote host $ENV{REMOTE_ADDR} started a PureMessage installation, using
this CSM's installation defaults.
    
The remote host said that it is installing the following roles: $role

EOT
    $fh->close;
}
