# cgiwizard
# manages the process of displaying a wizard
#
package PureMessage::Manager::CgiWizard;
use Exporter;
use PureMessage::Manager::Widget;
@ISA = ('Exporter');
@EXPORT_OK = qw(w_run w_save_state w_get_state w_is_done w_raise_error);

use Data::Dumper;
use Carp;
use MIME::Base64();
use strict;

my $WIDGET = new PureMessage::Manager::Widget;


my %w_state;
my $w_data;
my $cgi;
my $w_title;
my $pageid = 0;
my $w_target = '';
my $cancel_target = '';
my $RESPONSE;

# sanity checking constant
my $SANE_WIZARD = 'sane_wizard';


# start the processing of the wizard
sub w_run{
	
	$w_title = shift;
	$w_target = shift;
	$cancel_target = shift;
	$w_data = shift;
	$cgi = shift;
	my $reason = shift;
	my $debug = shift;
        $RESPONSE = shift;
	unless($w_title, $w_target, $cancel_target, $w_data, $cgi, $reason){
		croak('All parameters to w_run() are required (except the debug paramter).');
	}

	if(!$cgi || ref($cgi) ne "HASH"){
		croak("CGI parameter must be a hash reference. Check your parameters.");
	}
		
	# get the page id, set it to 0 if this is the first page
	if($$cgi{'w_pageid'}){
		$pageid = $$cgi{'w_pageid'};
	}else{
                $pageid = 0;
	}


	unserialzeState();
	# sanity checks
	if($pageid > 0 && !$w_state{'w_s'}){
		$$reason = "Environment not sane. Misssing sanity state value.";
		return 0;
	}

	if($pageid > 0 && $w_state{'w_s'}->{'one'} ne $SANE_WIZARD){
		$$reason = "Environment not sane. Sanity state value did not match.";
		return 0;
	}
	
	if($debug){
		# dump stuff into the CGI hash so the caller could display what they want at
		# the appropriate time
		$$cgi{'start_WIZARD_state'} = Dumper \%w_state;
		$$cgi{'start_WIZARD_data'} = Dumper $w_data;
		$$cgi{'start_WIZARD_pageid'} = $pageid;
		
	}
	# Are we going forward or backwards? 
	my $rc = 0;
	if($$cgi{'w_next'} || $$cgi{'w_next.x'} || $$cgi{'w_finish'} || $$cgi{'w_finish.x'}){
		$rc = do_next($reason);
	}elsif($$cgi{'w_back'} || $$cgi{'w_back.x'}){
		$rc =  do_back($reason);
	}else{ 
		# This must be the first request so load the first wizard page
		$rc = do_first($reason);
	}
	if($debug){ # if we made it this far
		# dump stuff into the CGI hash so the caller could display what they want at
		# the appropriate time
		$$cgi{'end_WIZARD_state'} = Dumper \%w_state;
		$$cgi{'end_WIZARD_data'} = Dumper $w_data;
		$$cgi{'end_WIZARD_pageid'} = $pageid;
		
	}
	return $rc;
}

sub w_save_state{
	my $pagekey = shift;
	my $pagedata = shift;
	$w_state{$pagekey} = $pagedata;
}

sub w_get_state{
	my $pagekey = shift;
	if(!$pagekey){
		return \%w_state;
	}
	return $w_state{$pagekey};
}

sub w_is_done{
	return ($pageid > $#{$w_data});
}

sub w_raise_error{
	return $RESPONSE->raise_error(@_);
}

sub do_first{
	my $reason = shift;
	return display_page($reason);
}


sub do_next{
	my $reason = shift;

	# call this page's action function
	my $pageaction = $w_data->[$pageid]{'action'};
	if(!$pageaction){
		$$reason = "Missing page action function for page ID: $pageid";
		return 0;
	}
	unless(&$pageaction($cgi, $reason)){
		
		# oops, there was an error so redisplay the current page
		return display_page($reason);
	}
	
	# display the next page (if this is the last page, just return true)
	$pageid ++;
	unless(w_is_done()){
		return display_page($reason);
	}
	return 1;
	
}

sub do_back{
	my $reason = shift;


	# display the previous page
	$pageid --;
	return display_page($reason);
	
}

sub display_page{
	
	my $reason = shift;
	$RESPONSE->title($w_title);
	$RESPONSE->error_dlg();
	
	# set the form action target (user may have selected a different target)
	if($w_data->[$pageid]{'target'}){
		$w_target = $w_data->[$pageid]{'target'};
	}
	$RESPONSE->print("<form action=\"$w_target\" method=\"post\"\n>");

	# print the page header, and the wizard dialog header.	
	$RESPONSE->print($WIDGET->section_header($w_data->[$pageid]{'title'}));
	
	# if this ever gets implemented with cookies, call this before the theme_section_header().
	# for now, make sure it is after the start of the form so that it inculded in the post.
	serializeState();
	
	# now we need to display the content of the next page.
	my $rc = 1;
	my $pagedisplay = $w_data->[$pageid]{'display'};
	$rc = &$pagedisplay($cgi, $reason);
	$RESPONSE->print("<input type=\"hidden\" name=\"w_pageid\" value=\"$pageid\">\n");
	$RESPONSE->print("<table width=\"100%\" cellspacing=\"0\  cellpadding=\"0\" border=\"0\">\n");	
	$RESPONSE->print("<tr><td width=\"100\"></td><td>\n");
        my $tb;
	# is this the last page, first page or an in between page?
	if($pageid == 0){ # first page
		if($pageid == $#{$w_data}){ # first and last page?
			# FIXME: using standard form buttons until images are available
			$tb = $RESPONSE->ToolBar(
                                                         buttons=> [[
                                                                   { type=>'image',href=>$cancel_target,
                                                                     icon=>'cancel',text=>'Cancel',
                                                                   },
                                                                   { type=>'image',name=>'w_finish',
                                                                     icon=>'save',text=>'Finish',
                                                                     value=>'w_finish',
                                                                   },
                                                                   ]],
                                        );

		}else{
			$tb = $RESPONSE->ToolBar(
                                                         buttons=> [[
                                                                   { type=>'image',href=>$cancel_target,
                                                                     icon=>'cancel',text=>'Cancel',
                                                                   },
                                                                   { type=>'image',name=>'w_next',
                                                                     icon=>'continue',text=>'Next',
                                                                     value=>'w_next',
                                                                   },
                                                                   ]],
                                        );


		}
	}elsif($pageid >= $#{$w_data}){ # last page
		$tb = $RESPONSE->ToolBar(
                                                 buttons=> [[
                                                           { type=>'image',href=>$cancel_target,
                                                             icon=>'cancel',text=>'Cancel',
                                                           },
                                                           { type=>'image',name=>'w_back',
                                                             icon=>'back',text=>'Back',
                                                             value=>'w_back',
                                                           },
                                                           { type=>'image',name=>'w_finish',
                                                             icon=>'save',text=>'Finish',
                                                             value=>'w_finish',
                                                           }
                                                           ]],
                                        );


	}else{ # in between page
		# print the wizard dialog footer.
		# FIXME: using standard form buttons until images are available
		$tb = $RESPONSE->ToolBar(
                                                 buttons=> [[
                                                           { type=>'image',href=>$cancel_target,
                                                             icon=>'cancel',text=>'Cancel',
                                                           },
                                                           { type=>'image',name=>'w_back',
                                                             icon=>'back',text=>'Back',
                                                             value=>'w_back',
                                                           },
                                                           { type=>'image',name=>'w_next',
                                                             icon=>'continue',text=>'Next',
                                                             value=>'w_next',
                                                           },
                                                           ]],
                                        );


	}
	$RESPONSE->print("</td></tr></table>\n");
	$RESPONSE->add_part($RESPONSE->ThemeSectionFooter(toolbar=>$tb));
	$RESPONSE->print("</form>\n");
	return $rc;
	
	
}

sub serializeState{
	# serialize the state data into a form variable.
	
	# make sure we store a basic sanity value
	$w_state{'w_s'} = {'one'=>$SANE_WIZARD};
	
	# turn our nested hash structure into a string like:
	# pagekey=datakey:value;datakey2=value2;pagekey2= ....
	# where value and value2 are base64 encoded.
	#
        my $state_data;
	foreach my $key(keys(%w_state)){
	    $state_data .= "$key=";
		foreach my $subkey(keys(%{$w_state{$key}})){
			$state_data .= "$subkey:";
			$state_data .= MIME::Base64::encode_base64($w_state{$key}->{$subkey},''); 
		}
		$state_data .= ";";
	}
	$RESPONSE->print("<input type=\"hidden\" name=\"w_s\" value=\"$state_data\">\n");
	}

sub unserialzeState{
	if($$cgi{'w_s'}){
		my @st1 = split(';',$$cgi{'w_s'});
		foreach(@st1){
			/(\w*)=(\w*):(\w*)/;
			$w_state{$1}->{$2} = MIME::Base64::decode_base64($3);
		}
	}
}
1;
__END__
=pod

PureMessage::Manager::CgiWizard

Package for easily creating wizards in CGI to be run in the webmin environment. It is assumed that the wizard is a simple linar process that
the user follows step by step. The wizard handls the logic of displaying the next and previous pages,


=head1 

w_run($wizard_title, $wizard_target, $cancel_target, \@wizard_data, \%cgi_data, \$reason)

The I<wizard_run()> function is the primary external interface to the wizard library that most CGI's will use. This function starts the execution of
every wizard page and is passed the structure that contains the information that the wizard needs.

=head2

B<IMPORTANT NOTE:> It is important that users of this package B<don't> out put any information to the STDOUT stream (responses to the browser) before 
the I<wizard_run()> function call is made. The I<wizard_run()> function manipulates the HTTP response headres (maily for cookies) and will not work
properly if the headers have already been sent.


The I<wizard_run()> function expects a parameter that specifies the title that is displayed on the wizard page, a parameter that declares where each
CGI POST should go to, a parameter that declares where the cancel CGI POST should go, another that defines the structure of the wizard, 
one that that contains all the CGI variables and a reference to a scalar that will be used to set any error information if an error occurs. 

The wizard structure is an array reference where each array element is a hash reference that describes an individual page.

	$wizard = [{'title'=>'page 1 title', 'action'=>\&pg1_action, 'display'=>\&pg1_display}, 
	{'title'=>'page 2 title', 'action'=>\&pg2_action, 'display'=>\&pg2_display, 'target'=>'alldone.cgi'},};

The hash elements are:

=over

=item title

This is the text that is displayed on the header area of the wizard dialog.

This is similar to the section title on other pages. This can be free-form text or it can be a text key into the I<$text{}> array for the current language.

=item action

This is a reference to a subroutine that is called when this page is being advanced on to the next page before the page is displayed. This subroutine
can save any state varialbes, perform input validation or perform any other action that depends on the information input on this page.

If this subroutine returns boolean false value (0, '', undef) then the current page is redisplayed with an error message. It is up to this function to
make any raise_error() calls.

The I<action> function must not write any data to STDOUT. This funciton is called before any HTTP headers have been sent. Any data sent to STDOUT
in this function will cause a page error.

=item display

This is a reference to a subroutine that is called when the wizard page is about to be displayed. This subroutine is responsible for
displaying the appropriate HTML that will be placed in side the wizard dialog. An error in this function will lead to an error message
being displayed in the wizard dialog. Each page is responsible for generating its own display.

=item target

This value is optional. If this value is set it is used to define where the next CGI POST will go to when the B<Next> or B<Done> buttons are pressed.
This is only used on the last page. If this is used on any other page then the Back button will not work.

=back

The expected prototype for the I<action> and I<display> functions is:

	sub yourfuntion(\%cgi_data, \$reason)
	
Where I<cgi_data> is a hash ref that contains all the CGI POST/GET variables and I<reason> is used by the display to tell the wizard why an error occured.
The wizard will use this message in an error box on the appropriate screen.

This function returns a boolean true value on success and a false value on failure. A false return value meant that the wizard was unable to complete
what it is supposed to do. 

=head1

w_save_state(\%data, "pagekey")

This function is used to save state data that will be needed later in the wizard process. The caller passes a reference to a hash structure 
(I<%data>) that will be encoded and stored or retreival with I<w_get_state()>. The hash structure only supports sclar values. The I<"pagekey"> 
variable is used to identify which page this data is associated with. This ensures that each pages' data is separate.


=head1

w_get_state("pagekey")

This funciton is used to retrieve satate data for a particular page. The I<"pagekey"> matches the same pagekey that was set in I<w_save_data>. If 
I<"pagekey"> is undef, I<w_get_state()> returns the entire state structure. The format of this structure is:

$state = {'pagekey1'=>{'data1'=>'some data', 'data2'=>'some more data'}, 'pagekey2'=>{'data1'=>'some data', 'data2'=>'some more data'}};

=head1

w_is_done()

This returns true if the wizard has completed the last page and false if it has not. This is useful to be called after w_run() has returned an you have
special processing that needs to occur after the wizard completes.

=head1

w_raise_error('error_code', [$error_data, $more_error_data, ...])

This function causes an error to be displayed on the next page that is loaded. This funciton is just a proxy for the standard theme raise_error(). Use this
implementation to ensure your code works with future updates.


=cut
