package PureMessage::Manager::ProgressPage;
use strict;
use Data::Dump;
use IO::Scalar;

sub new {
	my $class = shift;
	my $params = shift;
	return undef if not exists $params->{worker};
	my $self = bless { %{$params} }, $class;
	return $self;
}

sub run {
	my $self = shift;
	my $worker = $self->{worker};
	my $now = time;
	
	if (!defined($self->{pid} = fork)) {
		die "<br>\nunable to fork!<br>\n";
	}
	if ($self->{pid}) {
		$self->{progress_file} = "progress-$now-$self->{pid}.html" if not exists $self->{progress}->{file};
		$self->{url} = "./$self->{progress_file}";
		$self->{body} = 'Initialising...';
		$self->{title} = 'Initialising...';
		
		$self->{refresh} = 1;
		$self->update_progress();		
	
		$self->{content_type} = 'text/html';
		$self->{refresh} = 0;
		print $self->refresh_page();	
		exit;
	}
	
        # this disconnects us from the web server
	close STDIN;
	close STDOUT;
		
	open STDERR, ">filter.err" or die $!;
	select STDERR; $|=1;
	
	my $OUTPUT = new IO::Scalar \$self->{output};
	select $OUTPUT; $| = 1; 
	
	$self->{start} = time;
	$self->{pid} = $$;
	$self->{progress_file} = "progress-$now-$self->{pid}.html" if not exists $self->{progress}->{file};
	$self->{url} = "./$self->{progress_file}";
	$self->{refresh} = 1;
	
	&{$worker}($self);
}

sub update_progress {
	my $self = shift;
	my $body = shift;
	
	$self->{body} = $body if defined $body;
	
    open FP, ">$self->{progress_file}" 
			or $self->_die("$self->{progress_file}: $!");
	print FP $self->refresh_page();

	close FP;
}

sub cleanup {
	my $self = shift;
	my $body = shift;
	delete $self->{refresh};
	$self->update_progress($body);
	exit;
}

sub refresh_page {
        my $self = shift;
		my $headers = (exists $self->{content_type}) ? "Content-Type: $self->{content_type}\n\n" : "";
		my $refresh = (exists $self->{refresh}) ? "<meta HTTP-EQUIV=\"refresh\" CONTENT=\"$self->{refresh}; URL=$self->{url}\">" : "";
		my $style = (exists $self->{style}) ? $self->{style}  : "";
		my $stylesheet = (exists $self->{stylesheet}) ? "<link rel=\"stylesheet\" href=\"$self->{stylesheet}\" type=\"text/css\"/>" : "";
        return  <<EOT;
$headers
<html>
<head>
<title>$self->{title}</title>
$refresh
$stylesheet
$style	
</head>
<body>
$self->{body}
</body>
</html>
EOT
}


1;
