package PureMessage::Manager::RPCResponse::Services;
use PureMessage::Service;
use strict;
our  @ISA = ('PureMessage::Manager::RPCResponse');

sub snapshot {
    my $self = shift;
    my @services = PureMessage::Service->Available;
    my $data = {};
    for my $s (@services) {
        if ( $data->{$s->type} ) {
            # mixed status, e.g. one milter is running and another one is not
            $data->{$s->type} = -1 if $data->{$s->type} != $s->running();
        }
        else {
            $data->{$s->type} = $s->running;
        }
    }
    $self->dispatch($data);
}

sub stop {
    my $self = shift;
    my $type = shift;
    my $name = shift;
    my @services = PureMessage::Service->Available;
    my $run = 0;
    for my $s (@services) {
        next if $type and $s->type and $s->type ne $type;
        next if $name and $s->name and $s->name ne $name;
        $s->stop(bg=>1) if $s->running;
        $run = 1;
    }
    $self->dispatch(($run) ? "Ok" : "Error");
}

sub start {
    my $self = shift;
    my $type = shift;
    my $name = shift;
    my @services = PureMessage::Service->Available;
    my $data = {};
    my $run = 0;
    for my $s (@services) {
        next if $type and $s->type and $s->type ne $type;
        next if $name and $s->name and $s->name ne $name;
        $s->start(bg=>1) if not $s->running;
        $run = 1;
    }
    $self->dispatch(($run) ? "Ok" : "Error");
}

sub restart {
    my $self = shift;
    my $type = shift;
    my $name = shift;
    my @services = PureMessage::Service->Available;
    my $data = {};
    my $run = 0;
    for my $s (@services) {
        next if $type and $s->type and $s->type ne $type;
        next if $name and $s->name and $s->name ne $name;
        $s->restart(bg=>1);
        $run = 1;
    }
    $self->dispatch(($run) ? "Ok" : "Error");
}

sub status {
    my $self = shift;
    my $type = shift;
    my $name = shift;
    my @services = PureMessage::Service->Available;
    for my $s (@services) {
        next if $type and $s->type and $s->type ne $type;
        next if $name and $s->name and $s->name ne $name;
        my $data = $s->status();
        $data->{type} = $s->type;
        $data->{name} = $s->name;
	$data->{pid} = '' unless ($data->{pid});
        $self->dispatch($data);
    }
}

sub doc {
    my $self = shift;
    $self->dispatch("Manage PureMessage services.");
}

1;
