package PureMessage::Manager::RPCResponse::Subscription;
use PureMessage::Version qw ($PREFIX);
use ActiveState::File::Atomic;
use PureMessage::Manager::RPCRequest;
use PureMessage::List;
use PureMessage::Map;
use ActiveState::Config qw(:path);
                    
use strict;
our @ISA = ('PureMessage::Manager::RPCResponse');
my @listmeta = qw(name description precious source match_type comparator);
my @mapmeta  = qw(name description source match_type);
my $listfile = $PREFIX . "/etc/lists.conf";
my $mapfile  = $PREFIX . "/etc/maps.conf";

sub Status {
    my $self = shift;
    $self->raise_error("No files in status request") unless @_;
    foreach ( @_ ) {
        my (%file, $list, $map, $obj, $objtype, $objname);
	if ( $_ =~ ":" ) { # check if sender specified the object type
	    ($objtype, $objname) = split ":", $_;
	    $objtype = "" unless ( $objtype =~ /^(file|map|list)$/ );
	} 
	else {
	    $objname = $_;
	    $objtype = "";
	}
	if ( $objtype !~ /^file$/ ) { # check meta-information about this object
	    $objname =~ s[^\/?etc\/?][]; # remove "etc/"
	    eval { $list = PureMessage::List->Available($objname) };
	    eval { $map  = PureMessage::Map->Available($objname) };
	    if ( defined($map) && $map ) {
		$obj = $map;
		$obj->{obj_type} = $objtype || "map"; # caller expects the obj_type is ALWAYS set
	    } elsif ( defined($list) && $list ) {
		$obj = $list;

                # sync status checking requires that all the
                # meta fields be present
                foreach my $meta_field (@listmeta) {
                    unless (exists $obj->{$meta_field}) {
                        $obj->{$meta_field} = undef;
                    }
                }

		$obj->{obj_type} = $objtype || "list"; # caller expects the obj_type is ALWAYS set
	    }
	}
	my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime,$ctime);
	# a list has file and list info, a map has file and map info.
	if ( defined($obj) ) {
	    my ($otype, $osource) = split ":", $obj->{source};
	    if ( $otype =~ /^file$/i ) {
		($obj->{path} = $osource) =~ s[^$PREFIX\/?][]; # remove $PREFIX
		$obj->{source} =~ s[^(file:)$PREFIX\/etc\/][$1]; # reset this to compare with server
		$osource = "$PREFIX/$osource" if ( -f "$PREFIX/$osource" and ! -f $osource );
		$osource = "$PREFIX/etc/$osource" if ( -f "$PREFIX/etc/$osource" and ! -f $osource );
		($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime,$ctime) = stat($osource);
		$obj->{modified} = $mtime;
		$obj->{created} = $ctime;
		$obj->{size} = $size;
		$obj->{signature} = $size ? _checksum($osource) : "None";

		$self->dispatch($obj);
	    }
	    else {
		$self->raise_error("$objname list/map not found");
	    }
	} # either we have a "file" or the list or map is not defined yet on this host
	else {
	    $file{path} = $objname;
	    ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime,$ctime) = stat("$PREFIX/$file{path}");
	    $file{modified} = $mtime;
	    $file{created} = $ctime;
	    $file{size} = $size;
	    $file{signature} = $size ? _checksum("$PREFIX/$file{path}") : "None";
	    $file{obj_type} = $objtype || "file"; # caller expects the obj_type is ALWAYS set
	    
	    $self->dispatch(\%file);
	}
    }
}

sub Update {
    my $self = shift;
    my $publ = shift;
    my $host = shift; # XXX || automatically obtain remote address from ENV
    my %ldat = @_; # get any list/map meta-data
    while (my($l,$info) = each %ldat) {
	my($nm,$lkey) = split ":", $l;
	$ldat{$nm}{$lkey} = $info;
    }
    my ($i, $obj, @metadata, $conf);
    my $att = $self->{REQUEST}{attachments};
    foreach my $a ( values %$att ) {
	my $a_name = $a->{name};
        next unless ( ($a_name eq "file") || ($a_name eq "list") || ($a_name eq "map") );
        next unless $a->{filename};
        next unless $a->{location};
        my $dest = $PREFIX."/".$a->{filename};
	unless ( -f $dest ) {
	    open(F, ">$dest");
	    close F;
	}
	# XXX check that there are no relative paths
	# ...
	my $at = ActiveState::File::Atomic->new( $dest, 
						 writable => 1,
						 backup_ext => '.publ',
						 rotate => 1, );
	my $tf = $at->tempfile;
	open (my $af, "<$a->{location}");
	while ( <$af> ) {
	    print $tf $_;
	}
	unlink $a->{location};
	$at->commit_tempfile();

	if ( $a_name eq "file" ) {
	    $self->dispatch({FILE=>$a->{filename}, MODIFIED=>(stat($dest))[9], STATUS=>'Ok'});
	    $i++;
	} # list/map data updated, now update meta-data
	elsif ( ($a_name eq "list") || ($a_name eq "map") ) { 
	    unless ( -f $listfile ) {
		open(F, ">$listfile");
		close F;
	    }
	    unless ( -f $mapfile ) {
		open(F, ">$mapfile");
		close F;
	    }
	    if ($a_name eq "list") {
		@metadata = @listmeta;
		$conf = PureMessage::List->Config(writable => 1);
	    } elsif ($a_name eq "map") {
		@metadata = @mapmeta;
		$conf = PureMessage::Map->Config(writable => 1);
	    }
	    (my $objname = $a->{filename}) =~ s[^\/?etc\/?][];
	    for my $item ( @metadata ) {
		if ( exists($ldat{$objname}{$item}) ) {
		    path_value($conf->root, "$a_name#$objname.$item", $ldat{$objname}{$item});
		}
	    }
	    $conf->save;
	    undef($conf);
	    my $err;
	    if ($a_name eq "list") {
		$conf = PureMessage::List->Config();
		eval { $obj = PureMessage::List->Available($objname) }; # list exists?
	    } elsif ($a_name eq "map") {
		$conf = PureMessage::Map->Config();
		eval { $obj = PureMessage::Map->Available($objname) }; # map exists?
	    }
	    if ( defined($obj) && !$@ ) { 
		my %lconf = $conf->as_hash();
		for my $item ( @metadata ) {
		    if ( exists($ldat{$objname}{$item}) && 
			 ($lconf{$a_name}{$objname}{$item} ne $ldat{$objname}{$item}) ) {
			 $err .= ("Could not update $a_name: $objname::$item to -" . 
				  $ldat{$objname}{$item} . "- instead of *" .
				  $lconf{$a_name}{$objname}{$item} . "?\n");
		    }
		}
		undef($conf);
		if ( $err ) {
		    $self->dispatch({FILE=>$a->{filename}, MODIFIED=>(stat($dest))[9], STATUS=>"$err"});
		} else {
		    $self->dispatch({FILE=>$a->{filename}, MODIFIED=>(stat($dest))[9], STATUS=>'Ok'});
		}
		$i++;
	    }
	    else {
		$self->raise_error("Could not add $a_name: $objname meta-data.");
	    }
	} else {
	    $self->raise_error("My object type is $a_name which doesn't seem to be a file, list or map"); 
	}
    }
    $self->raise_error("No files found in request") unless $i;
}

sub doc {
    my $self = shift;
    $self->dispatch("RPC Service: subscribe to remote PureMessage configurations");
}

sub _checksum {
    my $file = shift;
    require Digest::MD5;
    my $ctx = Digest::MD5->new;
    open(my $F, $file);
    $ctx->addfile($F);
    return $ctx->hexdigest;
}


1;
