package PureMessage::Manager::Response::Backup;
use base 'Exporter';
use vars qw(@EXPORT_OK);
our @EXPORT_OK = qw(Dialog);
use PureMessage::Version qw($PREFIX);
use PureMessage::Manager::Widget;

# XXX move messages to LANG

sub Dialog {
    my $r = shift; # response object
    my $file = shift;
    return unless $file;
    $file =~ s{^$PREFIX/*} {};
    my $title = shift || "";
    my $extra_query = shift || "";
    my @menu;
    if ( my $file = $r->{REQUEST}->query('backup') ) {
        if ( -f "$PREFIX/$file" ) {
            my $backup = $file . ".bak" . time();
            $backup =~ s{/(?!.*/)} {/.}; # add dot in from of the file name (after last / in the path)
            my @args = ("cp", "$PREFIX/$file", "$PREFIX/$backup");
            if ( system(@args) == 0 ) {
                $r->print_notification("The backup for $title has been successfully created.");
            }
            else {
                $r->raise_error("Can't create a backup $backup: $?");
            }
        }
        else {
            $r->raise_error("Can't create a backup: file $file is not found");
        }
    }
    elsif ( $file = $r->{REQUEST}->query('ok_restore') ) {
        if ( -f "$PREFIX/$file" ) {
            my $orig = get_name($file);
            my @args = ("cp", "$PREFIX/$file", "$PREFIX/$orig");
            if ( system(@args) == 0 ) {
                $r->print_notification("The backup has been successfully restored.");
            }
            else {
                $r->raise_error("Can't restore from: $?");
            }
        }
        else {
            $r->raise_error("Can't restore from backup: file $file is not found");
        }
    }
    elsif ( $file = $r->{REQUEST}->query('restore') or $file = $r->{REQUEST}->query('bakdiff') ) {
        my @buttons;
        push @buttons, { type=>'image', icon=>'ok',text=>'Ok',href=>"?ok_restore=$file&$extra_query"};
        push @buttons, { type=>'image', icon=>'cancel',text=>'Cancel',href=>"?cancel&$extra_query"};
        push @buttons, { type=>'text', text=>'Diff', href=>"?bakdiff=$file&$extra_query"}
            unless $r->{REQUEST}->query('bakdiff');
        show_diff($r, $file) if $r->{REQUEST}->query('bakdiff');
        $r->dialog( style => 'dlg_info',
            msg => "Do you want to restore the $title configuration from the backup made on "
                    . get_title($file) . " ?",
            title=>"Confirmation required",
            buttons => [ \@buttons ],
            );
        $r->send();
        exit;
    }
    menu($r, $file, $title, $extra_query);
}

sub menu {
    my $r = shift;
    my $file = shift;
    my $title = shift || "";
    my $extra_query = shift || "";
    my $glob = "$file.bak*";
    $glob =~ s{/(?!.*/)} {/.};
    
    foreach (sort {$b cmp $a} glob "$PREFIX/$glob") {
        my $title = get_title($_);
        next unless $title;
        if ( $i >= 5 ) {
            # delete old backups
            unlink ($_);
            # make the rotate configurable ? XXX
        }
        else {
            s{^$PREFIX/*} {};
            push @menu, { type=>'text', text=>$title, href=>"?restore=$_&$extra_query" };
            $i++;
        }
    }
    push @menu, { type=>'custom', text=>"(no backups)" }
        unless @menu;
    push @menu, { type=>'custom', text=>"" };
    my $top_tb = button({type=>'image', 
                         icon=>'add_plus',
                         text=>"Create", 
                         href=>"?backup=$file&$extra_query" }, style=>"reverse");
    $r->add_taskmenu({ title => "Backups", name=>'backup', 
                       buttons => [ \@menu ],
                       tb => $top_tb });
}

sub get_title {
    my $file = shift;
    $file =~ /\.bak(\d+)/;
    my $stamp = $1;
    return "" unless $stamp;
    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime($stamp);
    my $title = sprintf("%04d-%02d-%02d %02d:%02d:%02d%s",
                        $year+1900, $mon+1, $mday, $hour, $min, $sec, "");
    return $title;
}

sub show_diff {
    my $r = shift;
    my $file = shift;
    my $orig = get_name($file);
    open(F, "diff $PREFIX/$orig $PREFIX/$file |");
    my @code = <F>;
    my $html = "";
    foreach (@code) {
        my $color = /^\</ ? "#A21E1E" : ( /^\>/ ? "#127719" : "#000000" );
        $html .= "<font color='$color'>$_</font>";
    }
    if ( $html ) {
        $r->print("<b>Changes:</b><br><PRE>$html</PRE>");
    }
    else {
        $r->print_notification("Diff results: Files are identical");
    }
}

sub get_name {
    my $file = shift;
    $file =~ s{/\.(?!.*/)} {/};
    $file =~ s{\.bak\d*$} {};
    return $file;
}

1;
