package PureMessage::Manager::Response::DashBars::Services;
use PureMessage::Service;
use ActiveState::Unix::Crontab qw(cron_list cron_parse cron_format cron_install);
use PureMessage::Manager::Widget;
use PureMessage::Manager::Lang;

# Figure out if the vscan scanner is running in daemon mode and ignore the error 
# if vscan is not installed.
my $no_daemons = 1;
eval { 
    require PureMessage::Policy::Virus::Daemon;
    $no_daemons = PureMessage::Policy::Virus::Daemon::no_daemons();
};

sub format_name {
    my ($type,$name) = @_;
    PureMessage::Manager::Lang::set_namespace("Services");
    return text($type . '_label', $type, $name);
}

sub Status {
    my $widget = shift;
    my $html = "";
    my @args = @_;
    my @services = sort { lc(format_name($a->type,$a->name)) cmp lc(format_name($b->type,$b->name)) } PureMessage::Service->Available;
    my @raw_lines = map { {cron_parse($_)} } ActiveState::Unix::Crontab->cron_list();
    PureMessage::Manager::Lang::set_namespace("Services");
    $html .= "<table width='100%' cellspacing=2>\n";
    foreach my $s ( @services ) {
        my $label = "";
        my $status = defined $s->running ? ($s->running ? "running" : "stopped") : "unknown";
        # hide the vscan service if there are enabled backends but none are running in daemon mode.
	next if $s->type eq 'vscan' && $no_daemons;

        if (text($s->type . '_label') ne $s->type . '_label') {
            $label .= button({ type=>'image',
                               text=>&text($s->type . '_label',$s->type,$s->name),
                               href=>&text($s->type . '_href',$s->type,$s->name),
                               icon=>$s->type . "_$status",
                            }, style=>"toolbar");
            $status = button({type => 'text', text => $status, href => &text($s->type . '_href',$s->type,$s->name) }, class => $status);
        } 
        else {
            $label .= button({ type=>'image',
                               text=> $s->name ? &text('generic_label2',$s->type,$s->name)
                                               : &text('generic_label',$s->type),
                               href=>&text('generic_href',$s->type,$s->name),
                               icon=>"generic__$status",
                            }, style=>"toolbar");
            $status = button({type => 'text', text=>$status, href=>&text('generic_href',$s->type,$s->name) }, class => $status);
        }
        $html .= "<tr><td nowrap=nowrap>$label</td><td width='100%'>$status</td></tr>";
    }
    $html .= "</table>\n";
}

1;
