package PureMessage::Manager::Response::Services::Manager;
use strict;
use PureMessage::Manager::Request;
use PureMessage::Manager::Config qw(%config %web_config);
use PureMessage::Manager::PasswordGenerator qw(mk_password);
use PureMessage::Manager::Users qw(:all $users_err_val $users_err_str);
use PureMessage::Version qw($VERSION $PREFIX);
use PureMessage::Manager::Lang;
use PureMessage::Manager::Modules;
use PureMessage::Manager::Widget;
use PureMessage::Manager::Response::Services;

our  @ISA = ('PureMessage::Manager::Response::Services');
my $locked = '*LK*'; # XXX write new disable_user() method instead

sub new {
    my $proto = shift;
    my $self = $proto->SUPER::new(@_);
    $self->add_taskmenu({ title => 'Manager Service',
                          buttons => [[
                                        { type=>'text', text=>'View status information', href=>'index.cgi'},
                                        { type=>'text', text=>'SSL configuration', href=>'edit_ssl.cgi'},
                                        { type=>'text', text=>'Define IP access control', href=>'edit_access.cgi'},
                                        { type=>'text', text=>'Configure Logging', href=>'edit_log.cgi'},
                                        { type=>'text', text=>'Browse Activity Log', href=>'activity.cgi'},
                                     ]], 
                        });
    $self->read_users_db();
    $self->{remote_user} = $ENV{'REMOTE_USER'};

    return $self;
}

sub send {
    my $self = shift;
    my @users_list;
    for ( sort keys (%{$self->{users}}) ) {
        push @users_list, { type=>'text', text=>$self->{users}{$_}{name} || $_, href=>"index.cgi?user=$_" };
    }
    push @users_list, { type=>'text', text=>"" };
    my $top_tb = button({type=>'image',
                         icon=>'add_plus',
                         text=>"New",
                         href=>"index.cgi?add_user=1" }, style=>"reverse");
    $self->add_taskmenu({ title => 'Administrators', buttons=> [\@users_list], tb=>$top_tb });

    my @groups_list;
    for ( sort keys (%{$self->{groups}}) ) {
        push @groups_list, { type=>'text', text=>"$_", href=>"index.cgi?group=$_" };
    }
    push @groups_list, { type=>'text', text=>"" };
    my $top_tb = button({type=>'image',
                         icon=>'add_plus',
                         text=>"New",
                         href=>"index.cgi?add_group=1" }, style=>"reverse");
    $self->add_taskmenu({ title => 'Groups', buttons=> [\@groups_list], tb=>$top_tb });
    $self->SUPER::send();
}

sub read_users_db {
    my $self = shift;
    eval {
        open_db({});
        my %users = list_users();
        $self->{users} = \%users;
        my %groups = list_groups();
        $self->{groups} = \%groups;
    };
    if ($@) {
        $self->raise_error("Error reading ACL table. $@");
    }
}


sub password_input {
    my $self = shift;
    return "<input class=stretch type=password name=$_[0] value='$_[1]'>";
}

sub enabled_combo {
    my $self = shift;
    my $enabled = shift;
    my $yes = $enabled ? "selected" : "";
    my $no = !$enabled ? "selected" : "";
    return <<EOT
<select class=stretch name=enabled>
    <option value="yes" $yes>Yes 
    <option value="no" $no>No
</select>
EOT
}

sub show_members {
    my $self = shift;
    my $gid = shift;
    my $html;
    my %members = list_group_members($gid);
    foreach ( sort keys %members ) {
        $html .= $_ . " (" . $self->{users}{$_}{name} . ")<br>";
    }
    return $html;
}

sub membership_ui {
    my $self = shift;
    my $uid = shift;
    my $html = "";
    my %my_groups = $uid ? list_user_membership($uid) : {};
    while ( my ($g, $group) = each (%{$self->{groups}}) ) {
        my $checked = $my_groups{$g} ? "checked" : "";
        $html .= "<input type=checkbox name=group value='$g' $checked>"
              .  "<a href='index.cgi?group=$g'>$g</a> group<br>";
    }
    return $html;
}

sub permissions_ui {
    my $self = shift;
    my %opts = @_;
    require PureMessage::Manager::Modules;
    my $m = PureMessage::Manager::Modules->new();
    my %perms = $m->get_permissions();
    my %has;
    my $ui;
    if ( $opts{uid} ) {
        %has = get_user_acl($opts{uid});
    }
    elsif ( $opts{gid} ) {
        %has = get_group_acl($opts{gid});
    }
    else { return "" }

    $ui .= "<input type=checkbox name=perm value='*' "
        . ( $has{'*'} ? "checked " : "" )
        .  "><b>All modules</b><br>";

    foreach ( sort keys %perms ) {
        my $checked = $has{$_} ? "checked" : "";
        $ui .= <<EOT;
<input type=checkbox name=perm value='$_' $checked>$perms{$_}{description}<br>
EOT
    }
    return $ui;
}

sub save_group {
    my $self = shift;
    my $REQUEST = shift;
    my $gid = $REQUEST->query('gid');
    my $g = $REQUEST->query('group');
    my $perm = $REQUEST->query('perm') || [];
    my %in = @_;
    if ( ! $gid ) {
        return $self->raise_error("You must specify a group name.");
    }
    my $group = $self->{groups}{$gid} || undef;
    if ( $group ) {
        # attempt to create existing group
        unless ( $g ) {
            $self->raise_error("Group '$gid' already exists.");
#            delete $in{name};
            return 0;
        }
    }
    else {
        # add new group
        return $self->raise_error($users_err_val)
            unless add_group($gid);
        my %groups = list_groups();
        $self->{groups} = \%groups;
        $group = $self->{groups}{$gid};
    }
    $perm = [ $perm ] unless ref($perm) eq "ARRAY";
#    use Data::Dumper; print Dumper($perm);
    return $self->raise_error($users_err_val)
        unless grant_group_acl($group->{group}, $perm);
    $self->save_changes();
}

sub save_user {
    my $self = shift;
    my $REQUEST = shift;
    my $uid = $REQUEST->query('uid');
    my $u = $REQUEST->query('user');
    my $new = $REQUEST->query('new');
    my $currpass = $REQUEST->query('currpass');
    my $pass = $REQUEST->query('pass');
    my $cpass = $REQUEST->query('cpass');
    my $enabled = $REQUEST->query('enabled');
    my $name = $REQUEST->query('name');
    my $perm = $REQUEST->query('perm') || [];
    my $group = $REQUEST->query('group') || [];
    if ( ! $uid ) {
        return $self->raise_error("You must specify a login name.");
    }
    elsif ( $pass ne $cpass ) {
        return $self->raise_error("Passwords do not match.");
    }
    elsif ( $pass !~ /\w/ and $enabled ne "no" and $pass !~ /^\*+$/) {
        return $self->raise_error("Please specify the password.");
    }
    my $user = $self->{users}{$uid} || undef;
    
    # disable account
    $pass = $locked if $enabled eq "no";
    
    if ( $user ) {
        # existing user
        if ( ! $u ) {
            # attempt to create existing user
            return $self->raise_error("User '$uid' already exists.");
        }
        elsif ( $pass !~ /^\*+$/ ) {
            my $cur_user = $ENV{REMOTE_USER};
            my $passwd_mismatch_error = "Your password is incorrect. ";
            if ($uid ne $cur_user) { 
                $passwd_mismatch_error .= " Note that this is the\n" .   
                    "password for the logged-in user, not the user\n" .
                    "being updated.";
            }
            my $real_pass = $self->{users}->{$cur_user}->{password};

            return $self->raise_error($passwd_mismatch_error)
                unless ($real_pass eq mk_password($currpass, $real_pass));
            
            return $self->raise_error($users_err_str)
                unless reset_password($user->{uid}, $pass);
        }
        $user->{name} = $name;
    }
    else {
        # new user
        return $self->raise_error($users_err_str)
            unless add_user($uid, $pass, name=>$name);
        my %users = list_users();
        $self->{users} = \%users;
        $user = $self->{users}{$uid};
    }
    
    # update/assign permissions
    $perm = [ $perm ] unless ref($perm) eq "ARRAY";
    return $self->raise_error($users_err_val)
        unless ( grant_user_acl($user->{uid}, $perm) );
  
    # update/assign membership
    $group = [ $group ] unless ref($group) eq "ARRAY";
    return $self->raise_error($users_err_val)
        unless ( change_groups($user->{uid}, $group) );
    
    $self->save_changes();
    return 1;
}

sub delete_user {
    my $self = shift;
    my $REQUEST = shift;
    my $uid = $REQUEST->query('uid');
    if ( ! $uid ) {
        return $self->raise_error("You must specify a user name.");
    }
    my $user = $self->{users}{$uid};
    return $self->raise_error("User '$uid' not found")
        unless $user;
    return $self->raise_error($users_err_val)
        unless del_user($user->{uid});
    $self->save_changes();
    return 1;
}

sub delete_group {
    my $self = shift;
    my $REQUEST = shift;
    my $gid = $REQUEST->query('gid');
    if ( ! $gid ) {
        return $self->raise_error("You must specify a group name.");
    }
    my $group = $self->{groups}{$gid};
    return $self->raise_error("Group '$gid' not found")
        unless $group;
    return $self->raise_error($users_err_val)
        unless del_group($group->{group});
    $self->save_changes();
    return 1;
}

sub save_changes {
    my $self = shift;
    commit_db();
    open_db();
    my %users = list_users();
    $self->{users} = \%users;
    my %groups = list_groups();
    $self->{groups} = \%groups;
    $self->print_notification("Users database has been updated. <a href='?restart=1'>Restart</a> the Manager to activate the changes.");
}

sub is_special_user {
    my $self = shift;
    my $uid = shift;
    return 1 if $uid eq $self->{remote_user};  # myself
    return 1 if $uid =~ /\bnetwork\b/; # cluster "manager"
    return 0;
}

sub is_special_group {
    my $self = shift;
    my $gid = shift;
    return 1 if $gid =~ /\badmin\b/; # "admin" group should always exist
    return 0;
}

sub select_users {
    my $self = shift;
    my $users = shift;
    my %users;
    my $js = "OnChange=\"this.form.uall.checked=0;\"";
    map { $users{$_}++ } split(/\s+/, $web_config{'logusers'});
    my $html = "<select multiple size=10 name=users class=stretch $js>\n";
    foreach ( keys %{$self->{users}} ) {
        $html .= sprintf "<option %s>%s\n", $users{$_} ? 'selected' : '', $_;
    }
    $html .= "</select>";
    $html .= sprintf "<br><input type=checkbox name=uall %s>%s",
             $users ? "" : "checked", text('log_uall');
    return $html;
}

sub select_modules {
    my $self = shift;
    my $modules = shift;
    my $js = "OnChange=\"this.form.mall.checked=0;\"";
    my $html = "<select multiple size=10 name=modules class=stretch $js>\n";
    my %mods;
    map { $mods{$_}++ } split(/\s+/, $modules);
    my $m = PureMessage::Manager::Modules->new();
    my $mods = $m->get_modules();
    foreach ($m->get_module_names()) {
        next if /Dashboard/; # there will never(?) be logging happening in this module 
        $html .= sprintf "<option value=%s %s>%s\n",
                $_, $mods{$_} ? 'selected' : '', $mods->{$_}{'title'} || $_;
    }
    $html .= "</select>";
    $html .= sprintf "<br><input type=checkbox name=mall %s>%s",
             $modules ? "" : "checked", text('log_mall');
    return $html;
}

sub parse_logline
{
    my $self = shift;
    my $line = shift;
    if ($line =~ /^(\d+)\.(\S+)\s+\[.*\]\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+"([^"]+)"\s+"([^"]+)"\s+"([^"]+)"(.*)/ ||
        $line =~ /^(\d+)\.(\S+)\s+\[.*\]\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)(.*)/) {
	my $rv = {  'time' => $1, 'id' => "$1.$2",
                    'user' => $3, 'sid' => $4,
                    'ip' => $5, 'module' => $6,
                    'script' => $7, 'action' => $8,
                    'type' => $9, 'object' => $10 };
	my %param;
	my $p = $11;
        while($p =~ /^\s*([^=\s]+)='([^']*)'(.*)$/) {
            if (defined($param{$1})) {
                $param{$1} .= "\0".$2;
            }
            else {
                $param{$1} = $2;
            }
            $p = $3;
        }
        foreach my $k (keys %param) {
            $param{$k} =~ s/%(..)/pack("c",hex($1))/ge;
        }
        $rv->{'param'} = \%param;
        return $rv;
    }
    else {
        return undef;
    }
}

