package PureMessage::Manager::Response::Services::Milter;
use strict;
use PureMessage::Manager::Request;
use PureMessage::Manager::Response::Services;
use PureMessage::Manager::Config qw(%config %web_config);
use PureMessage::Version qw($VERSION $PREFIX);

our  @ISA = ('PureMessage::Manager::Response::Services');

sub new {
    my $proto = shift;
    my $self = $proto->SUPER::new(@_);
    $self->add_taskmenu({ title => 'Milter Service',
                          buttons => [[
                                        { type=>'text', text=>'Milter Status', href=>'index.cgi'},
                                        { type=>'text', text=>'Milter Options', href=>'options.cgi'},
                                        { type=>'text', text=>'Activity Log', href=>'logs.cgi'},
                                     ]], 
                        });
    return $self;
}


=item hasConfigData
  print "Milter is in pmx.conf" if hasConfigData($milterName);

Returns true if the milter appears in the %config hash. This is only
true if the milter is listed in the pmx.conf file.

=cut

sub hasConfigData {
  my $this = shift;
  my $milterName = shift;
  return exists $config{'milter'}->{$milterName};
}


=item getMilterNames

returns the names of all milters that are configured in pmx.conf
  
=cut

sub getMilterNames{
	return keys %{$config{'milter'}};
}


# Returns a formatted string displaying the PureMessage license information and
# copyright.
# FIXME: version string is not language independant
sub getVersionString{
	
	my $ver = PureMessage::Version::Version();
        my $b = $PureMessage::Version::BUILD;
	return "PureMessage v$ver build $b";
}
sub getVersionStringBasic{
	return PureMessage::Version::Version();
}


=item getPmxLicenseString

  my $text = getPmxLicenseString();

This sub will return a string representation of the license data that is found. 
The returned string is suitable for display to users. If non license data can be
found undef will be returned instead.

=cut

sub getPmxLicenseString {
  require PureMessage::_sys;
  my $lictext = PureMessage::_sys::lic_desc();
  return $lictext;
}


=item getMilterPort

  my $port = getMilterPort('Main');

Returns the port that is configured for this milter, or if the milter 
doesn't explicitly define one then the default port of: inet:3366@localhost.

=cut


my $DEFAULT_MILTER_PORT = 'inet:3366@localhost';

sub getMilterPort {
  my $this = shift;
  my $milter = shift;
  return $config{milter}->{$milter}->{port} || $DEFAULT_MILTER_PORT 
    if defined $config{milter}->{$milter};

  return;
}


# this will read the available filters from our configuration file if the file
# exists and has data
sub getConfiguredAvailableFilters {
        my $this = shift;
        require ActiveState::Config;
        my ($AS,$data);
        eval {
                $AS = new ActiveState::Config();
                $AS->parse_file("$config{prefix}/etc/manager/Services/Milter/add_milter.conf");
                my $temp = $AS->as_hash();
                # AS will return a hash ref if there are no filters in the desired
                # section
                $data = $temp->{'available-filters'} if ref($temp->{'available-filters'}) =~ 'ARRAY';
        };
        if ($@) {
                $this->raise_error('error_accessing_add_milter_configuration',$@);
                return ();
        }

        $data = [$data] unless ref($data) =~ 'ARRAY';
        return @{$data};
        
}

# returns an array ref of available filters
sub getAvailableFilters {
  my $this = shift;
  my @filters;
  my %found_filters;
  
  foreach my $filter ($this->getConfiguredAvailableFilters()) {
        next unless $filter;
        next if $found_filters{$filter};
        next unless $this->isInstalledModule($filter);
        push(@filters,$filter);
        $found_filters{$filter} = 1;
  }

  foreach my $key ( keys %{ $config{filter} } ) { 
    push(@filters,$key) unless $found_filters{$key};
    $found_filters{$key} = 1;
  };

  return \@filters;
}


# this sub will take a passed module name and return true if it can find a
# matching package somewhere in the @INC 

sub isInstalledModule {
        my $this = shift;
        my $module_name = shift;
        my $filename = $module_name;
        $filename =~ s#::#/#;

        my($realfilename,$result);
        foreach my $prefix (@INC) {
                my $realfilename = "$prefix/$filename.pm";
                return 1 if -f $realfilename;
        }

        return 0;
}



1;
