package PureMessage::Manager::Widget::RestartDlg;
use PureMessage::Manager::Widget::ToolBar;
use PureMessage::Manager::Widget;
use PureMessage::Manager::Lang;
use PureMessage::Service;
use strict;

our @ISA = qw(PureMessage::Manager::Widget);

sub new {
    my $CLASS = shift;
    $CLASS = 'PureMessage::Manager::Widget::RestartDlg';
    my $self = $CLASS->SUPER::new(@_);
    $self->{name} = 'restart_dlg';
    $self->{width} = '100%';
    return bless($self,$CLASS);
}

sub output {
    my $self = shift;
    my @msg;
    PureMessage::Manager::Lang::set_namespace("Services");
    # also accept query_ok from input to mean the same thing
    $self->{query_ok} = $self->{query_ok} || $self->{REQUEST}->query('query_ok');
    unless ($self->{query_ok}) {
        return "" if $self->{REQUEST}->query('start') 
                or $self->{REQUEST}->query('restart');
        return "" if $self->{REQUEST}{method} eq "GET" 
                and scalar($self->{REQUEST}->query_names()) > 0;
        # if it's a post though make the page specifically request we don't
        # display
        return "" if $self->{REQUEST}{method} eq "POST" 
                and grep { $_ eq 'query_ok' } $self->{REQUEST}->query_names() and ! $self->{REQUEST}->query('query_ok');
    }
    $self->read_input($self->{REQUEST});
    my $q = $self->result_data();
    # the value of restart will be the type|name of the service that should be
    # restarted
    my ($type,$name);
    if ($q->{restart}) {
        ($type,$name) = split(/\|/,$q->{restart});
        if ($type and $name) {
            foreach my $s (PureMessage::Service->Available($type)) {
                next unless $s->name eq $name;
                $s->restart(bg=>1);
                my $label = text($s->type . '_label',$s->type,$s->name);
                push(@msg,{ msg => text('restart_sent', $label), button => ''});
            }
        }
    }

    $self->{type} = 'milter' unless $self->{type};
    $self->{buttons} = '' unless $self->{buttons};
    foreach my $s (PureMessage::Service->Available()) {
        my $status = $s->status();
        # skip services we've already restarted
        next if $q->{restart} and $type and $name and $type eq $s->type and $name eq $s->name();
        if ($status->{running} and $status->{config} eq 'stale') {
            my ($label, $href);
            if (text($s->type . '_label')) {
                $label = text($s->type . '_label',$s->type,$s->name);
                $href = text($s->type . '_href',$s->type,$s->name);
            } 
            else {
                $label = $s->name ? text('generic_label2',$s->type,$s->name)
                                  : text('generic_label',$s->type);
                $href = text('generic_href',$s->type,$s->name);
            }
            my $buttons = PureMessage::Manager::Widget::ToolBar->new(
                                buttons => [[{ 
                                    type=>'image', icon=>'restart',
                                    text=>text('restart_now'), 
                                    name=>'restart',
                                    value => $s->type . "|" . $s->name() }]],
                                action =>  $ENV{SCRIPT_NAME}, # send data back to me
                                name => "restart",
                                embed => 1,
                              );
            push(@msg, { msg => text('restart_is_needed', $label), button => $buttons->output() });
        }
    }
    if ( @msg) {
        return format_output(@msg);
    }
    else {
        return "";
    }
}

sub format_output {
    my (@msg) = @_;
    my $txt;
    $txt =  '<table cellpadding=4 cellspacing=0 width="100%">';
    foreach my $row (@msg) {
    my ($left,$right) = ($row->{msg},exists($row->{button}) ? $row->{button} : '');
    $txt .= <<EOT;
<tr>
<td width="1" class="infoBox">
    <img src="/img/info.gif">
</td><td class="infoBox">$left
</td><td align="right" class="infoBox">
$right
</td>
</tr>
EOT
    }
    $txt .= '</table> <img src="/img/x.gif" height=4>';
    $txt;
}

1;
